import requests
import json
import logging
import configparser as cp
from time import sleep
from os import path
from sys import exit
from sophos_central_api_connector import sophos_central_api_polling as api_poll
from sophos_central_api_connector.config import sophos_central_api_config as api_conf


def send_to_splunk(api, splunk_events, splunk_creds, sourcetype_value, tenant_id, from_str, to_str, alerts_exists,
                   temp_exists, alertids_exists, poll):
    # Generate the Splunk URL to be used to send the data to.
    splunk_conf_path = api_conf.splunk_conf_path
    splunk_final_path = get_file_location(splunk_conf_path)
    splunk_conf = cp.ConfigParser()
    splunk_conf.read(splunk_final_path)
    main_splunk_url, channel = set_main_splunk_url()
    splunk_headers = {"Authorization": "Splunk {0}".format(splunk_creds)}

    # Check indexer acknowledgement settings
    request_ack = splunk_conf.get('splunk_hec', 'verify_ack_result')

    if api == "common":
        # if the api is common post the alerts to splunk
        logging.info("Length of splunk events passed to post_alerts: {0}".format(len(splunk_events)))
        post_alerts(from_str, temp_exists, poll, splunk_events, main_splunk_url, request_ack, channel, splunk_headers,
                    tenant_id)
    elif api == "endpoint":
        # if the api is endpoint post the inventory data to splunk
        logging.info("Length of splunk events passed to post_inventory: {0}".format(len(splunk_events)))
        post_inventory(splunk_events, main_splunk_url, request_ack, channel, splunk_headers, poll)
    else:
        logging.error("No api function available for: {0}".format(api))


def set_main_splunk_url():
    splunk_conf_path = api_conf.splunk_conf_path
    splunk_final_path = get_file_location(splunk_conf_path)
    splunk_conf = cp.ConfigParser()
    splunk_conf.read(splunk_final_path)
    # construct the main splunk url from the config information
    splunk_url = splunk_conf.get('splunk_hec', 'splunk_url')
    channel = splunk_conf.get('splunk_hec', 'channel')
    splunk_ack_enabled = splunk_conf.get('splunk_hec', 'splunk_ack_enabled')

    if splunk_ack_enabled == "1":
        # if the token has the splunk acknowledgement enabled we need to pass a channel GUID to be successful
        # this is provided from the splunk_config.ini file
        logging.info("Indexer Acknowledgement set on HEC token, applying correct URI")
        main_splunk_url = "{0}?channel={1}".format(splunk_url, channel)
    elif splunk_ack_enabled == "0":
        # if no splunk acknowledgement is set then simply use the base splunk url provided in the config
        logging.info("No Indexer Acknowledgement set on HEC token, applying correct URI")
        main_splunk_url = splunk_url

    return main_splunk_url, channel


def post_alerts(from_str, temp_exists, poll, spl_events, main_splunk_url, request_ack, channel, splunk_headers,
                tenant_id):
    if poll:
        # If polling is set prepare the events to be sent to Splunk
        logging.info("Prep alert events for polling")
        spl_events = api_poll.prepare_poll(spl_events, temp_exists, from_str)
    else:
        # if no polling parameter has been passed then this is skipped
        logging.info("No alert polling set")

    if spl_events is None:
        # if there are no events to process then we return to process the next tenant if any
        logging.info("spl_events is 'None'")
        return
    elif len(spl_events) == 0:
        # if there are no events to process then we return to process the next tenant if any
        logging.info("Length of spl_events is: 0")
        return
    else:
        # if there are events we run a for loop to send the events to splunk
        logging.info("Length of splunk events being passed to splunk: {0}".format(len(spl_events)))

    # Go through the list of events and attempt to send to Splunk
    send_count = 0
    logging.info("Iterating through events and sending to Splunk...")
    logging.info("Length of events sent: {0}".format(len(spl_events)))
    for spl_event in spl_events.values():
        # Extract the events id. tenant and raised at values
        logging.debug(spl_event)
        event_id = spl_event['event']['id']
        event_tenant_id = spl_event['event']['tenant']['id']
        raised_at = spl_event['event']['raisedAt']
        event_data = {event_id: {"raised_at": raised_at, "tenant_id": event_tenant_id}}
        spl_event = json.dumps(spl_event)
        send_count += 1
        # send the extracted information to splunk
        hec_response_code, failed_id = post_events_to_splunk(spl_event, splunk_headers,
                                                             main_splunk_url, send_count,
                                                             request_ack, event_data,
                                                             event_id, poll, channel)


def post_inventory(splunk_events, main_splunk_url, request_ack, channel, splunk_headers, poll):
    # Go through the list of events and attempt to send to Splunk
    send_count = 0
    raised_at = None
    logging.info("Iterating through events and sending to Splunk...")
    logging.info("Length of events sent: {0}".format(len(splunk_events)))
    for spl_event in splunk_events.values():
        # Extract the events id. tenant and raised at values
        logging.debug(spl_event)
        event_id = spl_event['event']['id']
        event_tenant_id = spl_event['event']['tenant']['id']
        event_data = {event_id: {"tenant_id": event_tenant_id}}
        spl_event = json.dumps(spl_event)
        send_count += 1
        hec_response_code, failed_id = post_events_to_splunk(spl_event, splunk_headers,
                                                             main_splunk_url, send_count,
                                                             request_ack, event_data,
                                                             event_id, poll, channel)


def post_events_to_splunk(spl_event, splunk_headers, main_splunk_url, send_count, request_ack, event_data, event_id,
                          poll, channel):
    logging.debug("Event data content in post: {0}".format(event_data))
    try:
        # attempt to send the event to splunk
        hec_res = requests.post(main_splunk_url, headers=splunk_headers, data=spl_event)
        logging.debug(hec_res.status_code)
        hec_data = hec_res.json()
        logging.debug(hec_data)
        hec_response_code = hec_data['code']
        hec_response_text = hec_data['text']
        logging.debug(hec_data['text'])
        # If the sending of the data is successful to Splunk evaluate whether to check indexer acknowledgement
        if hec_response_code == 0:
            # if the event is successfully sent then check if the polling parameter has been passed to add the event id
            logging.debug("EventID: '{0}' has been sent to splunk".format(event_id))
            failed_id = None
            if poll:
                # add the event data to the alert_ids.json
                api_poll.gen_idalerts(event_data)
            else:
                pass
            return hec_response_code, failed_id
        elif hec_response_code == 8 or hec_response_code == 9:
            logging.error("Event failed to send to Splunk with error: {0}".format(hec_response_text))
            logging.debug(splunk_headers)
            # if the server is busy then send for a retry
            hec_retry_code, failed_id = hec_post_retry(spl_event, main_splunk_url, splunk_headers, event_id)
            return hec_response_code, failed_id
        elif hec_response_code != 0:
            failed_id = True
            # send to failure
            event_failure(spl_event, poll, failed_id)
    except requests.exceptions.HTTPError as err_http:
        logging.error("HTTP Error:", err_http)
    exit(1)


def hec_post_retry(spl_event, main_splunk_url, splunk_headers, event_id):
    # We will only try to resend the event a max of three times
    logging.debug(splunk_headers)
    max_retries = 3
    retries = 0
    retry = True
    while retry is True:
        # Attempt to send the event to Splunk again
        logging.info("Retrying to send ID: '{0}' to splunk. Attempt: {1}".format(event_id, retries))
        sleep(1)
        retries += 1
        logging.debug("Retry: {0}".format(retries))
        hec_res = requests.post(main_splunk_url, headers=splunk_headers, data=spl_event)
        hec_retry_data = hec_res.json()
        hec_retry_code = hec_retry_data['code']
        hec_retry_text = hec_retry_data['text']
        logging.debug(hec_retry_data)
        if hec_retry_code == 0:
            # If successful set the retry to false to break out of the loop
            logging.info("Retry successfully sent to Splunk ID: '{0}'".format(event_id))
            retry = False
            failed_id = False
            return hec_retry_code, failed_id
        elif retries == max_retries:
            logging.error("ID: '{0}' failed to send to Splunk. "
                          "This will be sent to the event failure def".format(event_id))
            retry = False
            failed_id = True
            return hec_retry_code, failed_id
        elif hec_retry_code == 8 or hec_retry_code == 9:
            # Retry sending back to the start of the loop
            retry = True
        elif hec_retry_code != 8 or hec_retry_code != 9:
            # This is due to a major failure so we need to break out completely
            logging.error("Failed to send the event failing out: {0}".format(hec_retry_text))
            exit(1)


def event_failure(spl_event, poll, failed_id):
    poll_path = api_conf.poll_conf_path
    final_poll_conf_path = get_file_location(poll_path)
    # if the event failure is true check whether the polling config exists
    exists = path.isfile(final_poll_conf_path)
    if exists and poll:
        # if the polling config exists and the polling parameter is true then write failure to the config
        try:
            with open(final_poll_conf_path, 'r') as pa_file:
                poll_dict = json.load(pa_file)
                for item in poll_dict:
                    if item['last_run_success_bool'] in ['True']:
                        item['last_run_success_bool'] = 'False'
                    if item['failures_seen'] in ['True']:
                        item['failures_seen'] = 'True'
            with open(final_poll_conf_path, 'w') as pa_file:
                json.dump(poll_dict, pa_file, indent=2)
        except Exception as exc:
            print("Exception raised: {0}".format(exc))
    else:
        log_path = api_conf.logging_path
        final_logging_path = get_file_location(log_path)
        # Set the failed event to a file for retry later or just report
        with open(final_logging_path, 'r') as fail_file:
            failed_events = json.load(fail_file)

        # update the dictionary with the new event information that failed
        failed_events.update(failed_id)
        with open(final_logging_path, 'w') as new_fails:
            json.dump(failed_events, new_fails, ensure_ascii=False, indent=2)


def get_file_location(process_path):
    dir_name = path.dirname(__file__)
    final_path = "{0}{1}".format(dir_name, process_path)
    return final_path



from pathlib import PurePath

# Static URI Variables
auth_uri = 'https://id.sophos.com/api/v2/oauth2/token'
whoami_uri = 'https://api.central.sophos.com/whoami/v1'
tenants_ptr_uri = 'https://api.central.sophos.com/partner/v1/tenants'
tenants_org_uri = 'https://api.central.sophos.com/organization/v1/tenants'
alerts_uri = '/common/v1/alerts'
endpoints_uri = '/endpoint/v1/endpoints'
directory_uri = '/common/v1/directory'
settings_uri = '/endpoint/v1/settings'
livediscover_uri = '/live-discover/v1/queries'


# Default Value
max_alerts_page = 100
max_inv_page = 500
max_localsite_page = 100

# Regex Variables
uuid_regex = '[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[1-5][0-9a-fA-F]{3}-[89abAB][0-9a-fA-F]{3}-[0-9a-fA-F]{12}'

# Path locations
sophos_conf_path = PurePath("/config/sophos_config.ini")
splunk_conf_path = PurePath("/config/splunk_config.ini")
intelix_conf_path = PurePath("/config/intelix_config.ini")
misp_conf_path = PurePath("/config/misp_config.ini")
output_inv_path = PurePath("/output/get_inventory")
output_al_path = PurePath("/output/get_alerts")
output_ls_path = PurePath("/output/get_local_sites")
output_intx_path = PurePath("/output/intelix")
output_intx_del_path = PurePath("/output/intelix/delete_local_sites")
poll_conf_path = PurePath("/polling/poll_config.json")
poll_alerts_path = PurePath("/polling/alert_ids.json")
poll_temp_path = PurePath("/polling/temp_alert_ids.json")
poll_path = PurePath("/polling")
logging_path = PurePath("/logs/failed_events.json")
temp_path = PurePath("/tmp")



import logging
import requests


def delete_local_site(del_dict, tenant_url_data):
    logging.info("Delete selected sites from local site data")
    processed_del_dict = dict()

    for ten_id, ten_item in tenant_url_data.items():
        tenant_id = ten_id
        logging.info("Deleting local sites for tenant: {0}".format(tenant_id))
        for site_id, site_item in del_dict.items():
            tenant_ref = site_item['tenantId']
            if tenant_ref == tenant_id:
                orig_url = ten_item['orig_url']
                headers = ten_item['headers']
                ls_url = "{0}/{1}".format(orig_url, site_id)
                del_ls = requests.delete(ls_url, headers=headers)
                del_data = del_ls.json()
                del_status = del_data['deleted']
                del_date = del_ls.headers['date']
                logging.debug("Local Site ID: {0}, Deletion Status: {1}".format(site_id, del_status))
            processed_del_dict[site_id] = {"tenantId": site_item['tenantId'], "intelixRisk": site_item['intelixRisk'],
                                           "url": site_item['url'], "delStatus": del_status, "date": del_date}
        logging.info("Completed deletion of local sites for tenant: {0}".format(tenant_id))
    return processed_del_dict



import json
import logging
import os
import configparser as cp
from sophos_central_api_connector.config import sophos_central_api_config as api_conf
from sophos_central_api_connector import sophos_central_api_connector_utils as api_utils


def process_output(output, json_items, tenant_url_data, tenant_id, api, sourcetype_value):
    # verifies which output argument has been passed and directs it to the correct function for processing
    logging.info("Verifying output parameter passed")
    logging.debug(output)
    if output == "stdout":
        # the results will be sent to stdout to print out the results to the terminal
        process_output_stdout(json_items)
        # we return no events as no further action is required
        events = None
        return events
    elif output == "json":
        # json has been specified to output to file
        # the filename is constructed from the tenant name and id so the files are unique
        filename = tenant_url_data[tenant_id]['filename']
        process_output_json(json_items, filename, api)
        # we return no events as no further action is required
        events = None
        return events
    elif output == "splunk" or output == "splunk_trans":
        # the events need to be processed for sending the events to splunk
        events = process_output_splunk(json_items, output, sourcetype_value)
        # events are returned after processing
        return events
    elif None:
        pass


def process_output_json(json_items, filename, api):
    if api == "endpoint":
        logging.debug("JSON file output, appending inventory data")
        # a new folder is created to store the files
        inv_path = api_conf.output_inv_path
        final_inv_path = api_utils.get_file_location(inv_path)
        if not os.path.exists(final_inv_path):
            os.makedirs(final_inv_path)
        with open(os.path.join(final_inv_path, filename), "w", encoding='utf-8') as ep_file:
            json.dump(json_items, ep_file, ensure_ascii=False, indent=2)
    elif api == "common":
        al_path = api_conf.output_al_path
        final_al_path = api_utils.get_file_location(al_path)
        logging.debug("JSON file output, appending alerts data")
        # a new folder is created to store the files
        if not os.path.exists(final_al_path):
            os.makedirs(final_al_path)
        with open(os.path.join(final_al_path, filename), "w", encoding='utf-8') as ep_file:
            json.dump(json_items, ep_file, ensure_ascii=False, indent=2)
    elif api == "local-sites":
        ls_path = api_conf.output_ls_path
        final_ls_path = api_utils.get_file_location(ls_path)
        logging.debug("JSON file output, appending local-sites data")
        # a new folder is created to store the files
        if not os.path.exists(final_ls_path):
            os.makedirs(final_ls_path)
        with open(os.path.join(final_ls_path, filename), "w", encoding='utf-8') as ls_file:
            json.dump(json_items, ls_file, ensure_ascii=False, indent=2)
    elif api == "intelix":
        intx_path = api_conf.output_intx_path
        final_intx_path = api_utils.get_file_location(intx_path)
        logging.debug("JSON file output, intelix local site check")
        # a new folder is created to store the files
        if not os.path.exists(final_intx_path):
            os.makedirs(final_intx_path)
        with open(os.path.join(final_intx_path, filename), "w", encoding='utf-8') as intx_file:
            json.dump(json_items, intx_file, ensure_ascii=False, indent=2)
    elif api == "intelix_del":
        del_path = api_conf.output_intx_del_path
        final_intx_del_path = api_utils.get_file_location(del_path)
        if not os.path.exists(final_intx_del_path):
            os.makedirs(final_intx_del_path)
        with open(os.path.join(final_intx_del_path, filename), "w", encoding='utf-8') as intx_file:
            json.dump(json_items, intx_file, ensure_ascii=False, indent=2)


def process_output_stdout(json_items):
    logging.debug(type(json_items))
    if len(json_items) == 0:
        # if there are no events then this process is passed.
        pass
    else:
        # if there are events these are printed to the terminal
        for item in json_items.values():
            print(json.dumps(item, indent=2))


def process_output_splunk(json_items, output, sourcetype_value):
    logging.debug("Send to Splunk: Building dictionary")

    def splunk_output(spl_events):
        # No changes are required to the event. Information in the transforms will be used.
        processed_events = dict()
        event_count = 0
        logging.debug("Standard Splunk output selected.")
        for event in spl_events.values():
            # construct new event
            pre_event = {"event": event}
            event_count += 1
            # add the constructed event to the processed_events dictionary
            processed_events["{0}".format(event_count)] = pre_event
        # return the processed events
        return processed_events

    def splunk_trans_output(spl_events, transforms_source):
        # apply the new transform information to the events
        logging.debug("Transformation selected. Applying settings from config to event.")
        processed_events = dict()
        event_count = 0

        # load and read the config file
        logging.info("Parsing the Splunk Configuration")
        splunk_conf_path = api_conf.splunk_conf_path
        splunk_final_path = api_utils.get_file_location(splunk_conf_path)
        splunk_conf = cp.ConfigParser()
        splunk_conf.read(splunk_final_path)

        # splunk_trans config info
        host = splunk_conf.get('splunk_transform', 'host')
        source = splunk_conf.get('splunk_transform', 'source')
        sourcetype = splunk_conf.get('splunk_transform', 'sourcetype')

        for event in spl_events.values():
            logging.debug("Standard Splunk output selected.")
            # construct new event
            pre_event = {"host": host, "source": source, "sourcetype": "{0}{1}".format(sourcetype, transforms_source),
                         "event": event}
            event_count += 1
            # add the constructed event to the processed_events dictionary
            processed_events["{0}".format(event_count)] = pre_event
        return processed_events

    # Determine which splunk action is required before returning the events to be sent to splunk
    if output == "splunk":
        splunk_events = splunk_output(json_items)
    elif output == "splunk_trans":
        splunk_events = splunk_trans_output(json_items, sourcetype_value)

    return splunk_events


def process_output_temp(tmp_dict, filename):
    try:
        tmp_path = api_conf.temp_path
        final_tmp_path = api_utils.get_file_location(tmp_path)
        # noinspection PyPep8
        logging.info("Attempting to write file: {0}\{1}".format(final_tmp_path, filename))
        if not os.path.exists(final_tmp_path):
            os.makedirs(final_tmp_path)
        with open(os.path.join(final_tmp_path, filename), "w", encoding='utf-8') as tmp_file:
            json.dump(tmp_dict, tmp_file, ensure_ascii=False, indent=2)
    except IOError:
        logging.error("Unable to write file: {0}\{1}".format(final_tmp_path, filename))
        return False
    else:
        tmp_file.close()
        return True



import requests
import logging
import getpass
import configparser as cp
import sophos_central_api_connector.config.sophos_central_api_config as api_conf
import sophos_central_api_connector.sophos_central_api_tenants as api_tenant
import sophos_central_api_connector.sophos_central_api_connector_utils as api_utils
import sophos_central_api_connector.sophos_central_api_awssecrets as awssecret


def ten_auth():
    sophos_conf_path = api_conf.sophos_conf_path
    sophos_final_path = api_utils.get_file_location(sophos_conf_path)

    sophos_conf = cp.ConfigParser(allow_no_value=True)
    sophos_conf.read(sophos_final_path)
    secret_name = sophos_conf.get('aws', 'secret_name')
    region_name = sophos_conf.get('aws', 'region_name')

    if secret_name and region_name:
        try:
            # Pull the credentials from AWS Secret Manager and pass to initialise Sophos Central API
            aws_secret = awssecret.get_secret(secret_name, region_name)
            client_id = aws_secret['client_id']
            client_secret = aws_secret['client_secret']
            logging.info(
                "Values have been applied to the credential variables, attempt to initialize Sophos Central API")
        except Exception as aws_exception:
            # Return the exception raised from the aws secrets script
            raise aws_exception
    else:
        logging.info("No AWS creds set in config. Requesting client_id and client_secret")
        client_id = getpass.getpass(prompt="Provide Sophos Central API Client ID: ", stream=None)
        if client_id:
            client_secret = getpass.getpass(prompt="Provide Sophos Central API Client Secret: ", stream=None)
        else:
            logging.error("No Client ID provided. Exiting")
            exit(1)

    # Set authorisation and whoami URLs
    auth_url = api_conf.auth_uri
    whoami_url = api_conf.whoami_uri
    partner_url = api_conf.tenants_ptr_uri
    organization_url = api_conf.tenants_org_uri

    # Get Sophos Central API Bearer Token for authorisation
    sophos_access_token = get_bearer_tok(client_id, client_secret, auth_url)

    # Construct id_headers
    headers = validate_id_headers(sophos_access_token)

    # Lookup up the unique ID assigned to the business entity for Sophos Central API
    whoami_id, whoami_type, whoami_data = get_whoami_data(headers, whoami_url)

    # Obtain correct whoami uri/header based on the whoami type
    header_type, tenant_url = validate_whoami_type(whoami_type, whoami_data, partner_url, organization_url)

    # Construct tenant headers
    tenant_headers = api_tenant.gen_tenant_headers(headers, whoami_id, whoami_type, header_type)

    # Check and gather tenant information
    if whoami_type == "tenant":
        tenant_info = api_tenant.type_tenant(tenant_headers, whoami_id, tenant_url, sophos_access_token)
    else:
        tenant_info = api_tenant.get_tenant_info(headers, tenant_url, sophos_access_token)

    return tenant_info


def get_bearer_tok(client_id, client_secret, auth_url):
    # Call the auth section by passing the client_id and client_secret information
    logging.info("Beginning authorisation process")
    # construct url for obtaining access_token
    logging.info("Obtaining Sophos Central API Authorisation URI")
    # set the initial header information
    logging.info("Setting basic header information for Sophos Central API")
    # set the headers
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    data_tok = "grant_type=client_credentials&client_id={0}&client_secret={1}&scope=token".format(client_id,
                                                                                                  client_secret)

    # post the url and api key information to obtain access_token
    logging.info("Getting the bearer token for Sophos Central API")
    res = requests.post(auth_url, headers=headers, data=data_tok)
    res_code = res.status_code
    res_data = res.json()
    # Check the response and act accordingly
    if res_code == 200:
        # Send back the access token and headers
        sophos_access_token = res_data['access_token']
        logging.info("Successfully obtained the bearer token")
        return sophos_access_token
    else:
        # Failed to obtain a bearer token
        logging.error("Failed to obtain the bearer token")
        res_error_code = res_data['errorCode']
        res_message = "Response Code: {0} Message: {1}".format(res_code, res_data['message'])
        return None, res_message, res_error_code


def validate_id_headers(sophos_access_token):
    # Check whether we have a valid bearer token
    if sophos_access_token[0] is not None:
        # apply the bearer token to the headers and start getting org_id
        logging.info("Applying the bearer token to the 'Authorization' headers")
        headers = {"Authorization": "Bearer {0}".format(sophos_access_token), "Accept": "application/json"}
        return headers
    else:
        # print the response code and message along with the error code
        logging.error("There is no valid bearer token present")
        logging.error(sophos_access_token[1])
        logging.error(sophos_access_token[2])
        exit(1)


def get_whoami_data(headers, whoami_url):
    # construct the url for obtaining the organisation uuid

    # send the request to get the whoami id details
    logging.info("Attempting to get whoami ID")
    try:
        res_whoami = requests.get(whoami_url, headers=headers)
        res_whoami_code = res_whoami.status_code
        whoami_data = res_whoami.json()
    except requests.exceptions.RequestException as res_exception:
        logging.error("Failed to obtain the whoami ID")
        res_whoami_error_code = whoami_data['error']
        logging.error(res_exception)
        logging.error("Err Code: {0}, Err Detail: {1}".format(res_whoami_code, res_whoami_error_code))
        exit(1)

    # check the response code and act accordingly
    if res_whoami_code == 200:
        whoami_id = whoami_data['id']
        whoami_type = whoami_data['idType']
        logging.info("Successfully obtained whoami ID")
        return whoami_id, whoami_type, whoami_data


def validate_whoami_type(whoami_type, whoami_data, partner_url, organization_url):
    # apply the uuid to the variable and add it to the correct header type
    if whoami_type == "partner":
        logging.info("whoami type is 'partner'")
        header_type = "X-Partner-ID"
        tenant_url = partner_url
        return header_type, tenant_url
    elif whoami_type == "organization":
        logging.info("whoami type is 'organization'")
        header_type = "X-Organization-ID"
        tenant_url = organization_url
        return header_type, tenant_url
    elif whoami_type == "tenant":
        logging.info("whoami type is 'tenant'")
        header_type = "X-Tenant-ID"
        tenant_url = whoami_data['apiHosts']['dataRegion']
        return header_type, tenant_url



import logging
import getpass
import json
import argparse as ap
import re
import configparser as cp
from sys import getsizeof
from sophos_central_api_connector import sophos_central_api_auth as api_auth
from sophos_central_api_connector import sophos_central_api_awssecrets as awssecret
from sophos_central_api_connector import sophos_central_api_connector_utils as api_utils
from sophos_central_api_connector.config import sophos_central_api_config as api_conf
from sophos_central_api_connector import sophos_central_api_output as api_output
from sophos_central_api_connector import sophos_central_api_live_discover as sld

# Global variables
api = "live-discover"
page_size = "250"


def main(params):
    log_level = "INFO"
    log_fmt = '%(asctime)s - %(levelname)s - %(message)s'
    log_name = log_level
    level = getattr(logging, log_name)
    logging.basicConfig(level=level, format=log_fmt, datefmt='%d/%m/%Y %I:%M:%S %p')

    logging.info("Start of Logging")

    # set params to variables
    tenant = params.tenant
    search_filter = params.filter
    search_variables = params.variables
    search_type = params.search_type
    search_val = params.search_input
    misp_attr = params.misp

    if misp_attr:
        misp_conf_path = api_conf.misp_conf_path
        misp_final_path = api_utils.get_file_location(misp_conf_path)

        misp_conf = cp.ConfigParser(allow_no_value=True)
        misp_conf.read(misp_final_path)
        secret_name = misp_conf.get('aws', 'secret_name')
        region_name = misp_conf.get('aws', 'region_name')
        misp_tok = misp_conf.get('aws', 'api_key')
        misp_url = misp_conf.get('url', 'misp_instance')

        # Get attributes for iocs to search
        misp_type = params.misp_type
        misp_val = params.misp_val

        if not misp_type or not misp_val:
            logging.error("You must specify a MISP type and value")
            exit(1)
        else:
            pass

        if misp_tok:
            try:
                # Pull the credentials from AWS Secret Manager and pass to initialise Sophos Central API
                misp_secret = awssecret.get_secret(secret_name, region_name)
                misp_tok = misp_secret['{0}'.format(misp_tok)]
                logging.info("MISP auth token applied")
            except Exception as aws_exception:
                # Return the exception raised from the aws secrets script
                raise aws_exception
            finally:
                attributes = sld.get_misp_attributes(misp_url, misp_type, misp_val, misp_tok, wildcard=True)
        else:
            logging.info("No AWS creds set in config. Requesting MISP API token")
            misp_tok = getpass.getpass(prompt="Provide MISP token: ", stream=None)
            if misp_tok:
                attributes = sld.get_misp_attributes(misp_url, misp_type, misp_val, misp_tok, wildcard=True)
            else:
                logging.error("No MISP token provided. Exiting")
                exit(1)

        if not attributes:
            logging.error("No attributes found, exiting")
            exit(1)
        else:
            logging.info("MISP attributes obtained")
            pass
    else:
        pass

    # format the filter variable to remove escape characters and pass as json
    if search_filter:
        try:
            search_filter = json.loads(re.sub('[\\?]', '', search_filter))
        except ValueError as err:
            logging.error("JSON malformed: {0}".format(search_filter))
            raise err
        else:
            pass
    elif search_type == "list":
        pass
    else:
        logging.error("No filter passed, A filter must be provided")
        exit(1)

    if search_variables and misp_attr:
        # Format the search date
        date_frmt = "{0}.000Z".format(search_variables[2])

        # esitmated size of variables
        ioc_size = getsizeof(attributes)
        if ioc_size < 1 or ioc_size > 5000:
            logging.error(
                "Size of IOC JSON must be in the range of 1 - 5000. Current estimated size is: {0}".format(ioc_size))
            exit(1)
        else:
            pass

        # Build JSON
        variables_json = [{"name": "Number of Hours of activity to search", "dataType": "text",
                           "value": "{0}".format(search_variables[0])},
                          {"name": "IOC JSON", "dataType": "text",
                           "value": attributes},
                          {"name": "Start Search From", "dataType": "dateTime", "value": "{0}".format(date_frmt)}]
    elif search_variables:
        # Format the search date
        date_frmt = "{0}.000Z".format(search_variables[2])

        # esitmated size of variables
        ioc_size = getsizeof(search_variables)
        if ioc_size < 1 or ioc_size > 5000:
            logging.error(
                "Size of IOC JSON must be in the range of 1 - 5000. Current estimated size is: {0}".format(ioc_size))
            exit(1)
        else:
            pass

        # Build JSON
        variables_json = [{"name": "Number of Hours of activity to search", "dataType": "text",
                           "value": "{0}".format(search_variables[0])},
                          {"name": "IOC JSON", "dataType": "text",
                           "value": search_variables[1]},
                          {"name": "Start Search From", "dataType": "dateTime", "value": "{0}".format(date_frmt)}]
    else:
        logging.info("No variables passed")
        variables_json = None

    # Auth and get tenant information
    tenant_info = api_auth.ten_auth()

    # Generate tenant url data
    tenant_url_data = api_utils.generate_tenant_urls(tenant_info, page_size, api, from_str=None, to_str=None)

    for key, value in tenant_url_data.items():
        # If a tenant has been passed in the CLI arguments it checks whether it exists in the tenants obtained
        if tenant == key:
            # The tenant passed has been found
            logging.info("Tenant ID passed: '{0}'".format(key))
            tenant_url_data = {key: value}
        else:
            pass

    # kick off live discover search
    if search_type == "list":
        query_data = sld.live_discover(tenant_url_data, search_type, search_val, search_filter, variables_json)
        for key, value in query_data.items():
            print(json.dumps(value, indent=2))
    else:
        ld_data, ep_data, res_data = sld.live_discover(tenant_url_data, search_type, search_val, search_filter,
                                                       variables_json)
        # output results to temp
        api_output.process_output_temp(ld_data, "search_data.json")
        api_output.process_output_temp(ep_data, "endpoint_data.json")
        api_output.process_output_temp(res_data, "result_data.json")

    logging.info("Script complete")
    exit(0)


if __name__ == "__main__":
    def set_bool_val(arg_val):
        if isinstance(arg_val, bool):
            return arg_val
        if arg_val.lower() in ('yes', 'true', 't', 'y', '1'):
            return True
        elif arg_val.lower() in ('no', 'false', 'f', 'n', '0'):
            return False
        else:
            raise ap.ArgumentTypeError('Boolean value expected')


    # Parse the various parameters to be used when calling the main script.
    parser = ap.ArgumentParser(formatter_class=ap.RawTextHelpFormatter,
                               description="This script allows you to pass IOC lists and perform queries in Live "
                                           "Discover and generate a report")
    parser.add_argument('-t', '--tenant', nargs='?', help="Allows to specify one tenant\nIf this argument is not "
                                                          "specified all tenants will apply.")
    parser.add_argument('-st', '--search_type', choices=["saved", "adhoc", "list"], required=True,
                        help="Must specify if the search to be run is a saved or adhoc search. Alternatively you "
                             "can list the saved queries available in the tenants (name, description, variables "
                             "and tenant)")
    parser.add_argument('-si', '--search_input', nargs='?', help="Enter input based on selected search type")
    parser.add_argument('-f', '--filter', nargs='?', help="Not setting a search filter will default to the pre-compiled"
                                                          " filter. You can set a custom JSON filter following the "
                                                          "documentation. \nEncapsulate your filter in single quotes, "
                                                          "''")
    parser.add_argument('-v', '--variables', nargs='+', help="Add the following values for the variables in order:\n"
                                                             "[0] Number of Hours of activity to search\n"
                                                             "[1] IOC JSON\n"
                                                             "[2] Start Search From")
    parser.add_argument('-m', '--misp', type=set_bool_val, nargs='?', const=True, default=False, help="Enable MISP "
                                                                                                      "attributes. "
                                                                                                      "Boolean")
    parser.add_argument('-mt', '--misp_type', choices=["eventid", "tag"], help="Specify whether you want to get "
                                                                               "attributes for an event ID or tag")
    parser.add_argument('-mv', '--misp_val', help="The value to pass according to the misp attribute type to search for")

    # parse the argument values to the arg variable
    args = parser.parse_args()

    # Go to main with the arguments
    main(args)



import intelix
import logging
import configparser as cp
from datetime import datetime
from ipaddress import ip_address
from sophos_central_api_connector import sophos_central_api_output as api_output, sophos_central_api_connector_utils \
    as api_utils
from sophos_central_api_connector.config import sophos_central_api_config as api_conf


def local_site_check(intelix_client_id, intelix_client_secret, request_data, tenant_id):
    logging.info("Checking local-sites against SophosLabs Intelix API")
    # dedup the urls from the data so we dont get site data twice
    output_data = dedup_url(request_data)

    # initiate authentication with intelix api
    logging.info("Authenticating with Intelix API")
    intx = intelix.client(intelix_client_id, intelix_client_secret)

    # iterate through the dictionary of urls and output
    intelix_dict = dict()

    logging.info("Pass URL data to Intelix for evaluation")
    for url_id, url_value in output_data.items():
        # check if the url_value is an ip
        try:
            ip_address(url_value)
            ip_val = True
        except:
            ip_val = False

        if ip_val:
            # send to ip_lookup
            intx.ip_lookup(url_value)
            ip_cat = intx.category
            maxRisk = get_ip_category_risk(ip_cat)
            intx_data = {"lookup_type": "ip", "requestId": intx.requestId, "ipCategory": intx.category, "riskLevel":
                maxRisk}
            intelix_dict[url_value] = intx_data
        elif not ip_val:
            # send_url_lookup
            intx.url_lookup(url_value)
            intx_data = {"lookup_type": "url", "requestId": intx.requestId,
                         "productivityCategory": intx.productivityCategory,
                         "securityCategory": intx.securityCategory, "riskLevel": intx.riskLevel}
            intelix_dict[url_value] = intx_data
        else:
            pass

    # keep record of results from intelix lookups
    date = datetime.now().strftime('%Y%m%d_%H%M%S')
    intx_filename = "{0}_intelix_results.json".format(date)
    logging.info("Saving JSON of Intelix results")
    api_output.process_output_json(intelix_dict, filename=intx_filename, api="intelix")

    # compare against site list
    compared_dict = site_comparison(intelix_dict, request_data, tenant_id)
    return compared_dict


def get_ip_category_risk(ip_cat):
    # get config information
    intelix_conf_path = api_conf.intelix_conf_path
    intelix_final_path = api_utils.get_file_location(intelix_conf_path)
    intelix_conf = cp.ConfigParser(allow_no_value=True)
    intelix_conf.read(intelix_final_path)

    if type(ip_cat) == str:
        maxRisk = intelix_conf.get('ip_lookup_risk', ip_cat)
    elif type(ip_cat) == list:
        ip_risk_dict = dict()
        for item in ip_cat:
            riskLevel = intelix_conf.get('ip_lookup_risk', item)
            ip_risk_dict[item] = riskLevel
        maxRisk = max(ip_risk_dict.values())

    # determine the risk as a string
    if maxRisk == "0":
        maxRisk = "UNCLASSIFIED"
    elif maxRisk == "1":
        maxRisk = "TRUSTED"
    elif maxRisk == "2":
        maxRisk = "LOW"
    elif maxRisk == "3":
        maxRisk = "MEDIUM"
    elif maxRisk == "4":
        maxRisk = "HIGH"

    return maxRisk


def dedup_url(request_data):
    logging.info("Deduplicating URLs from tenants")
    output_data = dict()
    for key, value in request_data.items():
        if value['url'] not in output_data.values():
            output_data[key] = value['url']

    return output_data


def site_comparison(intelix_dict, site_dict, tenant_id):
    # add details to the site dictionary from intelix output
    logging.info("Combine Intelix and Local-Sites information")
    combined_dict = dict()
    for site_key, site_val in site_dict.items():
        for intx_key, intx_val in intelix_dict.items():
            if site_val['url'] == intx_key:
                if intx_val['lookup_type'] == 'url':
                    intx_data = {"local-site": site_val}, {"intelix": {"intelixCategory": intx_val.setdefault(
                        'productivityCategory', 'null'), "intelixRisk": intx_val.setdefault('riskLevel', 'null'),
                        "intelixSecurity": intx_val.setdefault('securityCategory', 'null')}}
                    combined_dict[site_key] = intx_data
                elif intx_val['lookup_type'] == 'ip':
                    intx_data = {"local-site": site_val}, {"intelix": {"intelixCategory": intx_val.setdefault(
                        'ipCategory', 'null'), "intelixRisk": intx_val.setdefault('riskLevel', 'null')}}
                    combined_dict[site_key] = intx_data
                else:
                    pass
            else:
                pass

    date = datetime.now().strftime('%Y%m%d_%H%M%S')
    filename = "{0}_results_combined.json".format(date)
    logging.info("Saving JSON of Intelix and local-site results combined")
    api_output.process_output_json(combined_dict, filename=filename, api="intelix")
    return combined_dict


def intelix_report_info(intelix_results, intelix, intx_clean_level, intx_dry_run):
    # set main values
    report_dict = dict()
    high_risk = 0
    medium_risk = 0
    low_risk = 0
    trusted_risk = 0
    unclass_risk = 0
    total_risk = 0
    null_risk = 0

    for site_val in intelix_results.values():
        if intelix == "report":
            intx_data = site_val[1]['intelix']
            ls_data = site_val[0]['local-site']
            risk = intx_data.get('intelixRisk')
            tenant_id = ls_data.get('tenantId')
        elif intx_dry_run:
            risk = site_val.get('intelixRisk')
            tenant_id = site_val.get('tenantId')

        if risk == "HIGH":
            total_risk += 1
            high_risk += 1
        elif risk == "MEDIUM":
            total_risk += 1
            medium_risk += 1
        elif risk == "LOW":
            total_risk += 1
            low_risk += 1
        elif risk == "TRUSTED":
            total_risk += 1
            trusted_risk += 1
        elif risk == "UNCLASSIFIED":
            total_risk += 1
            unclass_risk += 1
        elif risk is None:
            total_risk += 1
            null_risk += 1
        else:
            total_risk += 1
            null_risk += 1
        report_data = {"Totals": {"High Risk": high_risk, "Medium Risk": medium_risk,
                                  "Low Risk": low_risk, "Trusted": trusted_risk,
                                  "Unclassified": unclass_risk, "NULL": null_risk,
                                  "Total": total_risk}}

    if intelix == "report":
        logging.info("Generating report for Intelix results")
        intx_filename = "{0}_intelix_report.json".format(tenant_id)
        report_dict.update(report_data)
        api_output.process_output_json(report_dict, filename=intx_filename, api="intelix")
        return report_dict
    elif intx_dry_run:
        date = datetime.now().strftime('%Y%m%d_%H%M%S')
        intx_filename = "{0}_{1}_{2}_dry_run_report.json".format(tenant_id, date, intx_clean_level)
        dryrun_dict = {**intelix_results, **report_data}
        api_output.process_output_json(dryrun_dict, filename=intx_filename, api="intelix")
        return dryrun_dict
    else:
        pass


def intelix_del_info(deletion_data, tenant_url_data):
    logging.info("Generate outcome of local site deletion")
    del_report = dict()
    deleted = 0
    failed = 0
    unknown = 0
    for ten_id, ten_item in tenant_url_data.items():
        tenant_id = ten_id
        for site_id, site_value in deletion_data.items():
            tenant_ref = site_value['tenantId']
            if tenant_ref == tenant_id:
                del_status = site_value['delStatus']
                if del_status == "True":
                    deleted += 1
                elif del_status == "False":
                    failed += 1
                else:
                    unknown += 1
        del_report[tenant_id] = {"Deleted": deleted, "Failed": failed, "Unknown": unknown}

    date = datetime.now().strftime('%Y%m%d_%H%M%S')
    del_filename = "{0}_deletion_details.json".format(date)
    del_report_filename = "{0}_deletion_report.json".format(date)
    api_output.process_output_json(deletion_data, filename=del_filename, api="intelix_del")
    api_output.process_output_json(del_report, filename=del_report_filename, api="intelix_del")


def test(intelix_client_id, intelix_client_secret, test_url):
    intx = intelix.client(intelix_client_id, intelix_client_secret)

    # check if the url_value is an ip
    try:
        ip_address(url_value)
        ip_val = True
    except:
        ip_val = False

    if ip_val:
        # send to ip_lookup
        intx.ip_lookup(test_url)
        intx_data = {"requestId": intx.requestId, "ipCategory": intx.category}
    elif not ip_val:
        # send_url_lookup
        intx.url_lookup(test_url)
        intx_data = {"requestId": intx.requestId, "productivityCategory": intx.productivityCategory,
                     "securityCategory": intx.securityCategory, "riskLevel": intx.riskLevel}


def prepare_del_dict(intx_clean_level, combined_results):
    # go through the combined list and extract the entries which match clean-level
    del_dict = dict()
    logging.info("Prepare the dictionary for local-site clean up")
    logging.info("Option to delete '{0}' has been passed".format(intx_clean_level))
    for site_key, site_val in combined_results.items():
        intx_data = site_val[1]
        ls_data = site_val[0]
        risk = intx_data['intelix']['intelixRisk']
        url = ls_data['local-site']['url']
        tenant_id = ls_data['local-site']['tenantId']
        if intx_clean_level == "ALL":
            # gather all of the valid entries bar unclassified and null. These should remain
            if risk == "HIGH" or risk == "MEDIUM" or risk == "LOW" or risk == "TRUSTED":
                del_dict[site_key] = {"tenantId": tenant_id, "intelixRisk": risk, "url": url}
            else:
                pass
        elif intx_clean_level == "HIGH":
            if risk == "HIGH":
                del_dict[site_key] = {"tenantId": tenant_id, "intelixRisk": risk, "url": url}
            else:
                pass
        elif intx_clean_level == "HIGH_MEDIUM":
            if risk == "HIGH" or risk == "MEDIUM":
                del_dict[site_key] = {"tenantId": tenant_id, "intelixRisk": risk, "url": url}
            else:
                pass
        elif intx_clean_level == "MEDIUM":
            if risk == "MEDIUM":
                del_dict[site_key] = {"tenantId": tenant_id, "intelixRisk": risk, "url": url}
            else:
                pass
        elif intx_clean_level == "LOW":
            if risk == "LOW":
                del_dict[site_key] = {"tenantId": tenant_id, "intelixRisk": risk, "url": url}
            else:
                pass
        else:
            pass

    return del_dict



import logging
import argparse as ap
import configparser as cp
from re import match
from sys import exit
from os import path
from sophos_central_api_connector import sophos_central_api_auth as api_auth, sophos_central_api_output as api_output, \
    sophos_central_api_get_data as get_api, sophos_central_api_polling as api_poll, \
    sophos_central_hec_splunk as splunk_hec, sophos_central_api_awssecrets as awssec, \
    sophos_central_api_connector_utils as api_utils, sophos_central_api_tenants as api_tenant, \
    sophos_central_api_intelix \
        as intx, sophos_central_api_delete_data as del_api
from sophos_central_api_connector.config import sophos_central_api_config as api_conf


# This will retrieve a list of local sites you have set in the global settings of your selected tenant(s)
def get_local_sites(tenant_info, output, page_size, tenant=None, intelix=None):
    # Set api type
    api = "local-sites"

    # Validate page size set
    page_size = api_utils.validate_page_size(page_size, api)

    # Generate urls for tenants
    tenant_url_data = api_utils.generate_tenant_urls(tenant_info, page_size, api, from_str=None, to_str=None)

    # Gather the categorization information
    cat_data = api_utils.gather_category_data(tenant_info)

    for key, value in tenant_url_data.items():
        # If a tenant has been passed in the CLI arguments it checks whether it exists in the tenants obtained
        if tenant == key:
            # The tenant passed has been found
            logging.info("Tenant ID passed: {0}".format(key))
            tenant_url_data = {key: value}
        else:
            # Does not match tenant data
            pass

    new_ls = dict()
    for ten_id, ten_item in tenant_url_data.items():
        # Pass the ten_url_data and gather the alerts
        logging.info("Gathering local site information")
        tenant_id = ten_id
        # get data information for the tenant in the loop
        json_data = get_api.get_data(tenant_url_data, page_size, tenant_id, api)
        for site_id, site_value in json_data.items():
            # grab the correct site category information
            cat_val = cat_data[site_value.setdefault('categoryId', 0)]
            # reconstruct dictionary using id as main key and include category if present
            cat_dict = {"tenantId": tenant_id, "categoryId": site_value.setdefault('categoryId', 'NA'), "categoryName":
                cat_val['name'], "tags": site_value.setdefault('tags', 'NA'), "url": site_value['url']}
            new_ls["{0}".format(site_value['id'])] = cat_dict
        # process data by the output parameter
        if not intelix:
            api_output.process_output(output, new_ls, tenant_url_data, tenant_id, api, sourcetype_value=None)
        else:
            pass

        # Completed gathering data for this tenant
        logging.info("Completed processing for Tenant ID: {0}".format(tenant_id))

    if intelix:
        # return the local-site information for processing
        return new_ls, tenant_id, tenant_url_data
    else:
        logging.info("Gathering local-site data complete")
        exit(0)


def intelix_check(tenant_info, output, page_size, tenant, intelix, intelix_client_id, intelix_client_secret,
                  intx_clean_level, intx_dry_run):
    output = None
    # gather the local site information to be sent to intelix
    local_site_data, tenant_id, tenant_url_data = get_local_sites(tenant_info, output, page_size, tenant, intelix)

    # query intelix data for whether sites are detected
    intelix_results = intx.local_site_check(intelix_client_id, intelix_client_secret, local_site_data, tenant_id)

    if intelix:
        if intelix == "report":
            # construct report for the local site information
            logging.info("Intelix reporting selected")
            intelix_report = intx.intelix_report_info(intelix_results, intelix, intx_clean_level, intx_dry_run)
            exit(0)
        elif intelix == "clean_ls":
            logging.info("Clean local-sites selected")
            # prepare local-sites for the risk level selected for intelix
            del_dict = intx.prepare_del_dict(intx_clean_level, intelix_results)
            # check if it is a dry-run, if so generate report
            if intx_dry_run:
                logging.info("Generating dry-run report for deleting '{0}' local-sites from Central"
                             .format(intx_clean_level))
                logging.info("This never includes sites which are 'UNCLASSIFIED' or 'NULL'")
                dry_run_report = intx.intelix_report_info(del_dict, intelix, intx_clean_level, intx_dry_run)
            else:
                logging.info("Commence deleting sites with '{0}' risk level local-sites from Central"
                             .format(intx_clean_level))
                logging.info("This never includes sites which are 'UNCLASSIFIED' or 'NULL'")
                intelix_clean = del_api.delete_local_site(del_dict, tenant_url_data)
                intx.intelix_del_info(intelix_clean, tenant_url_data)
                exit(0)


# This will retrieve inventory information from the tenants
def get_inventory(tenant_info, output, page_size, splunk_creds=None, tenant=None):
    # Generate urls for tenants
    api = "endpoint"
    # this value is added to the splunk config setting of the sourcetype to ensure they are labelled correct
    sourcetype_value = "inventory"

    # Validate page size set
    page_size = api_utils.validate_page_size(page_size, api)

    # set poll as none as polling is not available for inventory
    poll = None

    # Generate tenant url data
    tenant_url_data = api_utils.generate_tenant_urls(tenant_info, page_size, api, from_str=None, to_str=None)

    for key, value in tenant_url_data.items():
        # If a tenant has been passed in the CLI arguements it checks whether it exists in the tenants obtained
        if tenant == key:
            # The tenant passed has been found
            tenant_url_data = {key: value}
        else:
            # tenant doesnt match
            pass

    # Generate events and send them to output
    for ten_id, ten_item in tenant_url_data.items():
        # Pass the ten_url_data and gather the alerts
        logging.info("Attempting to get inventory information")
        tenant_id = ten_id
        # get data information for the tenant in the loop
        json_data = get_api.get_data(tenant_url_data, page_size, tenant_id, api)
        # process data by the output parameter
        events = api_output.process_output(output, json_data, tenant_url_data, tenant_id, api, sourcetype_value)

        if output == "splunk" or output == "splunk_trans":
            logging.info("Passing inventory to Splunk")
            # splunk argument passed, send the data to splunk
            splunk_hec.send_to_splunk(api, events, splunk_creds, sourcetype_value, tenant_id, from_str=None,
                                      to_str=None,
                                      alerts_exists=None, temp_exists=None, alertids_exists=None, poll=None)
            # Completed gathering data for this tenant
            logging.info("Completed processing for Tenant ID: {0}".format(tenant_id))
        else:
            pass
    else:
        # Completed gathering data for this tenant
        logging.info("Completed processing for Tenant ID: {0}".format(tenant_id))

    logging.info("Gathering inventory data complete")
    exit(0)


# this will retrieve alert information from the tenants
def get_alerts(tenant_info, output, poll, days, reset_flag, page_size, splunk_creds, tenant=None):
    # Set the type of api this is using
    api = "common"
    sourcetype_value = "alerts"

    # Validate page size set
    page_size = api_utils.validate_page_size(page_size, api)

    # Check if the polling log is available
    alerts_exists = path.isfile('sophos_central_api_connector/polling/poll_config.json')
    alertids_exists = path.isfile('sophos_central_api_connector/polling/alert_ids.json')
    temp_exists = path.isfile('sophos_central_api_connector/polling/temp_alert_ids.json')

    # If the number of days to check is set to None it will set the value to the default number of days: 1
    # It will also set the days_flag to False in order to validate args when checking polling in alerts
    if days is not None and alerts_exists is True and reset_flag is True:
        # Criteria has been met to use the day_flag for resetting polling
        day_flag = True
        logging.info("Reset flag passed with days and poll")
    elif days > 1 or alerts_exists is True:
        # defaults the days to 1
        days = 1
        day_flag = False
    else:
        # sets the day flag
        day_flag = True

    # Checks whether the polling arguement has been passed in order to calculate the to and from strings
    if poll:
        # Polling arguement has been passed
        if output == "json" or output == "stdout":
            # Will error is output is json or stdout. Only accepts splunk or splunk_trans
            logging.error("Polling not available for the output: {0}".format(output))
            exit(1)
        else:
            logging.info("Polling has been set")
            # splunk or splunk_trans has been passed. Pass the information to the polling alerts function
            to_str, from_str, reset_flag = api_poll.polling_alerts(alerts_exists, temp_exists, reset_flag, day_flag,
                                                                   days)
    else:
        logging.info("No polling passed")
        # No polling has been set so calulate from and to strings normally
        to_str, from_str = api_utils.calculate_from_to(days, poll_date=None)

    # Generate urls for tenants
    tenant_url_data = api_utils.generate_tenant_urls(tenant_info, page_size, api, from_str, to_str)

    for key, value in tenant_url_data.items():
        # If a tenant has been passed in the CLI arguements it checks whether it exists in the tenants obtained
        if tenant == key:
            # The tenant passed has been found
            tenant_url_data = {key: value}
        else:
            # Does not match tenant data
            pass

    for ten_id, ten_item in tenant_url_data.items():
        # Pass the ten_url_data and gather the alerts
        logging.info("Attempting to get alert information for tenant: {0}".format(ten_id))
        tenant_id = ten_id
        # Send the data to get data to gather the events
        json_data = get_api.get_data(tenant_url_data, page_size, tenant_id, api)
        if len(json_data) > 0:
            # There are events to process. Send to check the output
            events = api_output.process_output(output, json_data, tenant_url_data, tenant_id, api,
                                               sourcetype_value)
            if output == "splunk" or output == "splunk_trans":
                # output parameter has been passed with splunk or splunk_trans
                logging.info("Splunk output selected. Send events")
                # pass the event data to the HEC script to be processed and sent to Splunk
                splunk_hec.send_to_splunk(api, events, splunk_creds, sourcetype_value, tenant_id,
                                          from_str, to_str, alerts_exists, temp_exists,
                                          alertids_exists, poll)
                logging.info("Completed processing for Tenant ID: {0}".format(tenant_id))
            else:
                pass
    else:
        # No further data to process for the tenant
        logging.info("Completed processing for Tenant ID: {0}".format(tenant_id))

    if poll:
        # If the polling parameter has been passed need to finalise the config and events
        api_poll.finalise_polling(poll, to_str)

    logging.info("Gathering alert data complete")
    exit(0)


def print_tenant_info(tenant_info):
    logging.info("Printing out tenant information")
    print("\nYour tenant information is as follows:\n")
    # loop through each of the tenants and print its information to stdout
    for ten_id, ten_item in tenant_info.items():
        ten_name = ten_item['name']
        print("Tenant Name: {0}\nTenantID: {1}\n".format(ten_name, ten_id))
    exit(0)


def get_splunk_creds(splunk_final_path):
    # Initially set the Splunk flag and Splunk creds to None.
    splunk_aws_flag = None
    splunk_creds = None

    # load and read the splunk config
    logging.info("Reading Splunk configuration information")
    splunk_conf = cp.ConfigParser(allow_no_value=True)
    splunk_conf.read(splunk_final_path)
    splunk_use_aws = splunk_conf.get('splunk_aws', 'splunk_aws')
    splunk_ack_enabled = splunk_conf.get('splunk_hec', 'splunk_ack_enabled')
    splunk_verify_ack = splunk_conf.get('splunk_hec', 'verify_ack_result')

    # check authentication method
    logging.info("Splunk output has been set")
    if splunk_use_aws == "1":
        logging.info("Config setting set to use Splunk creds from AWS Secrets Manager")
        # Confirm that the check indexer acknowledgement has not been set without the config for ack_enabled being 0
        if splunk_verify_ack == "1" and splunk_ack_enabled == "0":
            logging.error(
                "To verify Splunk Index Acknowledgements, you must set the config 'splunk_ack_enabled' to '1'")
            exit(1)
        else:
            # Apply the splunk aws settings to variables in order to get aws secret information
            logging.info("Flag set to pull HEC token from AWS Secrets Manager. Getting config settings")
            token_key = splunk_conf.get('splunk_aws', 'token_key')
            secret_name = splunk_conf.get('splunk_aws', 'secret_name')
            region_name = splunk_conf.get('splunk_aws', 'region_name')
            # verify that the config is valid
            try:
                # Pull the credentials from AWS Secret Manager and add to splunk_creds variable
                logging.info("Attempting to get Splunk HEC token from AWS")
                aws_secret = awssec.get_secret(secret_name, region_name)
                splunk_token = aws_secret[token_key]
                # Verify that the token matches expected format
                aws_token_match = match(r"{0}".format(api_conf.uuid_regex), splunk_token)
                if aws_token_match is None:
                    # An exception will be raised if the token does not match the correct format
                    logging.error("Please ensure you have a valid HEC token in your AWS Secrets Manager")
                    exit(1)
                else:
                    logging.info("HEC token is in a valid format")
                    # splunk_creds = {'tok': '{0}'.format(splunk_token)}
                    splunk_creds = splunk_token
                    return splunk_creds
            except Exception("AWSException") as aws_exception:
                raise aws_exception
    elif splunk_use_aws == "0":
        logging.info("Config setting set to use static Splunk HEC token from config")
        logging.info("Getting static credentials for Splunk HEC from config")
        splunk_static_tok = splunk_conf.get('splunk_static', 'token')
        logging.info("Verying token passed is in the correct format")
        token_match = match(r"{0}".format(api_conf.uuid_regex), splunk_static_tok)
        # verify that the value in the static Splunk HEC token is valid
        if token_match is None:
            logging.error("Please ensure you have a valid HEC token in the splunk_config.ini")
            exit(1)
        else:
            logging.info("Token is in the correct format")
            splunk_creds = {'tok': '{0}'.format(splunk_static_tok)}
            return splunk_creds


def get_sophos_creds(sophos_auth, sophos_final_path):
    # load and read the sophos config file
    sophos_conf = cp.ConfigParser(allow_no_value=True)
    sophos_conf.read(sophos_final_path)

    if sophos_auth == "static":
        # Auth is static so the creds are pulled from the config
        logging.info("Static API credentials parameter, has been passed. Getting value from config")
        client_id = sophos_conf.get('static', 'client_id')
        client_secret = sophos_conf.get('static', 'client_secret')
        if int(len(client_secret)) == 0 or int(len(client_id)) == 0:
            # verifies that there is something in the variable
            logging.error("Please verify the static credentials are valid in config.ini")
            exit(1)
        else:
            logging.info("Values have been applied to the credential variables")
            return client_id, client_secret
    elif sophos_auth == "aws":
        # Creds are held in AWS, gather the config information
        logging.info("Attempting to get Sophos Central API credentials from AWS Secrets Manager")
        client_id_key = sophos_conf.get('aws', 'client_id_key')
        client_secret_key = sophos_conf.get('aws', 'client_secret_key')
        secret_name = sophos_conf.get('aws', 'secret_name')
        region_name = sophos_conf.get('aws', 'region_name')
        try:
            # Pull the credentials from AWS Secret Manager and pass to initialise Sophos Central API
            aws_secret = awssec.get_secret(secret_name, region_name)
            client_id = aws_secret[client_id_key]
            client_secret = aws_secret[client_secret_key]
            logging.info(
                "Values have been applied to the credential variables, attempt to initialize Sophos Central API")
            return client_id, client_secret
        except Exception as aws_exception:
            # Return the exception raised from the aws secrets script
            raise aws_exception
    else:
        # Invalid auth parameter has been passed
        logging.error(
            "No authentication or an incorrect authentication method has been specified.\nPlease run --help for "
            "further information")


def get_intelix_auth(sophos_auth, intelix_final_path):
    # load and read the sophos config file
    intelix_conf = cp.ConfigParser(allow_no_value=True)
    intelix_conf.read(intelix_final_path)

    if sophos_auth == "static":
        # Auth is static so the creds are pulled from the config
        logging.info("Static API credentials parameter, has been passed. Getting value from config")
        client_id = intelix_conf.get('static', 'client_id')
        client_secret = intelix_conf.get('static', 'client_secret')
        if int(len(client_secret)) == 0 or int(len(client_id)) == 0:
            # verifies that there is something in the variable
            logging.error("Please verify the static credentials are valid in config.ini")
            exit(1)
        else:
            logging.info("Values have been applied to the credential variables")
            return intelix_client_id, intelix_client_secret
    elif sophos_auth == "aws":
        # Creds are held in AWS, gather the config information
        logging.info("Attempting to get Sophos Intelix API credentials from AWS Secrets Manager")
        client_id_key = intelix_conf.get('intelix', 'client_id_key')
        client_secret_key = intelix_conf.get('intelix', 'client_secret_key')
        secret_name = intelix_conf.get('intelix', 'secret_name')
        region_name = intelix_conf.get('intelix', 'region_name')
        try:
            # Pull the credentials from AWS Secret Manager and pass to initialise Sophos Central API
            aws_secret = awssec.get_secret(secret_name, region_name)
            intelix_client_id = aws_secret[client_id_key]
            intelix_client_secret = aws_secret[client_secret_key]
            logging.info("Values have been applied to the Intelix credential variables")
            return intelix_client_id, intelix_client_secret
        except Exception as aws_exception:
            # Return the exception raised from the aws secrets script
            raise aws_exception


def get_file_location(process_path):
    dir_name = path.dirname(path.abspath(__file__))
    final_path = "{0}{1}".format(dir_name, process_path)
    return final_path


def main(args):
    # Assign parameters to variables
    log_level = args.log_level
    output = args.output
    days = args.days
    sophos_auth = args.auth
    poll = args.poll_alerts
    reset_flag = args.reset
    get = args.get
    tenant = args.tenant
    splunk_conf_path = api_conf.splunk_conf_path
    splunk_final_path = get_file_location(splunk_conf_path)
    sophos_conf_path = api_conf.sophos_conf_path
    sophos_final_path = get_file_location(sophos_conf_path)
    intelix_conf_path = api_conf.intelix_conf_path
    intelix_final_path = get_file_location(intelix_conf_path)
    intelix = args.intelix
    intx_clean_level = args.clean_level
    intx_dry_run = args.dry_run

    # Set authorisation and whoami URLs
    auth_url = api_conf.auth_uri
    whoami_url = api_conf.whoami_uri
    partner_url = api_conf.tenants_ptr_uri
    organization_url = api_conf.tenants_org_uri

    # Get sophos and intelix configs
    sophos_conf = cp.ConfigParser(allow_no_value=True)
    sophos_conf.read(sophos_final_path)

    # Set the level of the handler based on the value passed by the parameter
    if log_level is None:
        logging.disable(True)
    else:
        logging.disable(False)
        log_name = log_level
        level = getattr(logging, log_name)
        logging.basicConfig(level=level, format='%(asctime)s - %(levelname)s - %(message)s',
                            datefmt='%d/%m/%Y %I:%M:%S %p')

    logging.info("Start of logging")

    # Get Sophos Central API Creds and check for Splunk Creds if required
    client_id, client_secret = get_sophos_creds(sophos_auth, sophos_final_path)

    # Get Splunk credentials if required
    splunk_creds = None
    if output == "splunk" or output == "splunk_trans":
        splunk_creds = get_splunk_creds(splunk_final_path)

    # Get Sophos Central API Bearer Token for authorisation
    sophos_access_token = api_auth.get_bearer_tok(client_id, client_secret, auth_url)

    # Construct id_headers
    headers = api_auth.validate_id_headers(sophos_access_token)

    # Lookup up the unique ID assigned to the business entity for Sophos Central API
    whoami_id, whoami_type, whoami_data = api_auth.get_whoami_data(headers, whoami_url)

    # Obtain correct whoami uri/header based on the whoami type
    header_type, tenant_url = api_auth.validate_whoami_type(whoami_type, whoami_data, partner_url, organization_url)

    # Construct tenant headers
    tenant_headers = api_tenant.gen_tenant_headers(headers, whoami_id, whoami_type, header_type)

    # Check and gather tenant information
    if whoami_type == "tenant":
        tenant_info = api_tenant.type_tenant(tenant_headers, whoami_id, tenant_url, sophos_access_token)
    else:
        logging.info("Gather tenant information")
        tenant_info = api_tenant.get_tenant_info(headers, tenant_url, sophos_access_token)

    # Check which get parameter was passed and get that data
    logging.info("Check which parameters have been passed to main.")
    if get == "tenants":
        print_tenant_info(tenant_info)
    elif get == "inventory":
        logging.info("Inventory parameter passed.")
        logging.info("Checking page sizes passed through config")
        page_size = sophos_conf.get('page_size', 'inventory_ps')
        # begin the process to gather inventory information
        get_inventory(tenant_info, output, page_size, splunk_creds, tenant)
    elif get == "alerts":
        logging.info("Alerts parameter passed.")
        page_size = sophos_conf.get('page_size', 'alerts_ps')
        # begin the process to gather alert information
        get_alerts(tenant_info, output, poll, days, reset_flag, page_size, splunk_creds, tenant)
    elif get == "local-sites":
        logging.info("local-sites parameter passed.")
        page_size = sophos_conf.get('page_size', 'settings_ps')
        # begin the process to gather local-site information
        get_local_sites(tenant_info, output, page_size, tenant)
    elif intelix == "report":
        logging.info("Compare local-sites with SophosLabs Intelix classifications")
        intelix_client_id, intelix_client_secret = get_intelix_auth(sophos_auth, intelix_final_path)
        logging.info("Gather local-sites from Sophos Central tenants")
        page_size = sophos_conf.get('page_size', 'settings_ps')
        intelix_check(tenant_info, output, page_size, tenant, intelix, intelix_client_id, intelix_client_secret,
                      intx_clean_level, intx_dry_run)
    elif intelix == "clean_ls":
        if intx_clean_level:
            intelix_client_id, intelix_client_secret = get_intelix_auth(sophos_auth, intelix_final_path)
            page_size = sophos_conf.get('page_size', 'settings_ps')
            intelix_check(tenant_info, output, page_size, tenant, intelix, intelix_client_id, intelix_client_secret,
                          intx_clean_level, intx_dry_run)
            exit(0)
        else:
            logging.error("To run the clean_ls argument a clean_level must be provided. You can run a dry run to see"
                          " what the effect of command would be to prevent items being deleted from Central")
            exit(1)
    elif intelix == "test":
        intelix_client_id, intelix_client_secret = get_intelix_auth(intelix_final_path)
        url = "sophos.com"
        intx.test(intelix_client_id, intelix_client_secret, url)
    else:
        # invalid get parameter has been passed
        logging.error("Invalid --get parameter passed")
        exit(1)

    exit(0)


# this is executed if the script is called from the command line.
if __name__ == "__main__":
    # Parse the various parameters to be used when calling the main script.
    parser = ap.ArgumentParser(formatter_class=ap.RawTextHelpFormatter,
                               description="sophos_central_main.py:\n\nThis script utilises the Sophos Central API. "
                                           "In order to use the script you\nwill require a valid API key. To obtain a "
                                           "valid API key please reference\nthe documentation here: "
                                           "https://developer.sophos.com/intro")
    parser.add_argument('-a', '--auth', choices=['static', 'aws'], required=True,
                        help="This is a required option to specify which auth method to use.")
    parser.add_argument('-g', '--get', choices=['inventory', 'alerts', 'tenants', 'local-sites'],
                        help="This will set what information to get from tenants")
    parser.add_argument('-t', '--tenant',
                        help="Allows to specify one tenant\nIf this argument is not specified all tenants will apply.")
    parser.add_argument('-o', '--output', choices=['stdout', 'json', 'splunk', 'splunk_trans'], default="stdout",
                        help="Allows you to specify an output method for the data retrieved. If no option is selected"
                             "it will default to stdout.")
    parser.add_argument('-pa', '--poll_alerts', action='store_true',
                        help="N.B. This parameter only works with the '--get alerts' parameter:\nCalling this "
                             "parameter will set the value to true, in doing so it will maintain a config of when the "
                             "last\nquery was run and when it is next run will use this\ninformation to set the "
                             "relevant from flag to continue getting alerts from that point.")
    parser.add_argument('-d', '--days', type=int, choices=range(1, 91), default=1,
                        help="This is the number of days to go back and gather. Min is 1 and max number of days is 90.")
    parser.add_argument('-r', '--reset', action='store_true',
                        help="Setting this flag with the poll_alerts and days flags will reset the poll config")
    parser.add_argument('-ll', '--log_level', choices=['INFO', 'DEBUG', 'CRITICAL', 'WARNING', 'ERROR'],
                        help="Set logging level mode for more detailed error information, default is disabled")
    parser.add_argument('-i', '--intelix', choices=['report', 'clean_ls', 'test'], help="Generate "
                                                                                        "a check of local-sites and advise whether they are detected by SophosLabs")
    parser.add_argument('-cl', '--clean_level', choices=['ALL', 'HIGH', 'HIGH_MEDIUM', 'MEDIUM', 'LOW'],
                        help="Allows you to set the risk level for deleting local-sites from Sophos Central. It will "
                             "check Intelix for SophosLabs Risk assessment and delete on this value. Not what is set in"
                             "Sophos Central")
    parser.add_argument('-dr', '--dry_run', action='store_true', help="Allows you to run the local-sites clean option "
                                                                      "and report on what would have been deleted")

    # parse the argument values to the arg variable
    args = parser.parse_args()

    # Go to main with the arguments
    main(args)



import json
import logging
from datetime import datetime
from os import path, remove, makedirs
from sophos_central_api_connector import sophos_central_api_connector_utils as api_utils
from sophos_central_api_connector.config import sophos_central_api_config as api_conf


def polling_alerts(alerts_exists, temp_exists, reset_flag, day_flag, days):
    # Polling argument has been passed. Need to determine what the current state of polling is
    if not alerts_exists:
        # What to do if there is no polling config file present
        logging.info("Polling not previously run, creating config")
        # calculate the to and from dates
        to_str, from_str = api_utils.calculate_from_to(days, poll_date=None)
        # generate the new polling config
        alerts_exists = create_poll_config(to_str)
    elif alerts_exists is True and day_flag is True and reset_flag is True:
        # the correct parameters have been passed to reset the polling config. Understood that this may produce dupes
        logging.info("Polling config exits. Applying reset parameter")
        # calculate the to and from dates
        to_str, from_str = api_utils.calculate_from_to(days, poll_date=None)
        logging.debug("from: {0}, to: {1}".format(from_str, to_str))
        # recreate the polling config
        alerts_exists = create_poll_config(to_str)
        poll_tf_path = api_conf.poll_temp_path
        poll_temp_path = get_file_location(poll_tf_path)
        if alerts_exists and reset_flag:
            # If the polling config is set and the reset flag is passed. Prepare for the next run
            logging.info("Deleting alert_ids log")
            # delete the old alert_ids.json. This will remove the last events successfully sent
            remove(poll_alerts_path)
        if temp_exists and reset_flag:
            # If the temp file exists then it will be deleted if the reset flag is set
            logging.info("Deleting temp_alert_ids log")
            remove(poll_temp_path)
        logging.info("Setting reset_flag to 'False'")
        # set the reset_flag to false to avoid it being reset again on the next pass
        reset_flag = False
    elif alerts_exists is True and reset_flag is False:
        # Determines that polling has been run previously with no reset passed.
        logging.info("Poll config already exists and no reset flag has been set.")
        # determine when to poll alerts from to calculate the to and from datetime
        to_str, from_str = polling_from(days)
    elif alerts_exists is True and day_flag is True and reset_flag is False:
        # In order to pass the days flag with polling it must be run with the reset flag
        logging.error("You must raise a reset flag to pass days with poll_alerts when a poll config already exists")
        exit(1)
    elif alerts_exists is True and reset_flag is True and day_flag is False:
        # Required to pass days flag when reset and polling are passed to calculate correctly the datetimes
        logging.error("You must raise a days flag to pass reset with poll_alerts when a poll config already exists")
        exit(1)

    if alerts_exists:
        # only return the variables if the polling config exists
        return to_str, from_str, reset_flag
    else:
        exit(1)


def create_poll_config(to_str):
    # create the polling config
    poll_path = api_conf.poll_conf_path
    final_poll_conf_path = get_file_location(poll_path)
    poll_dir = api_conf.poll_path
    poll_dir = get_file_location(poll_dir)
    if not path.exists(poll_dir):
        makedirs(poll_dir)
    poll_dict = {'last_run_datetime': '{0}'.format(to_str),
                 'firsttime_run': 'True', 'last_run_success_bool': '', 'last_run_success_datetime': '',
                 'first_loop': 'True', 'failures_seen': 'False'}
    with open(final_poll_conf_path, 'w', encoding='utf-8') as pa_file:
        json.dump(poll_dict, pa_file, ensure_ascii=False, indent=2)

    # re-check whether the polling config is set in order to return correctly
    alerts_exists = path.isfile(final_poll_conf_path)
    return alerts_exists


def polling_from(days):
    # If the polling config is already available checks are made to determine the last state of the run to correctly
    poll_path = api_conf.poll_conf_path
    final_poll_conf_path = get_file_location(poll_path)
    # determine the from and to datetimes
    with open(final_poll_conf_path) as pa_file:
        poll_dict = json.loads(pa_file.read())
        if poll_dict['last_run_success_bool'] == 'False':
            # If the last run was not successful it uses the last successful datetime for the from datetime
            poll_date = datetime.strptime(poll_dict['last_run_success_datetime'], '%Y-%m-%dT%H:%M:%S.%fZ')
            to_str, from_str = api_utils.calculate_from_to(days, poll_date)
        elif poll_dict['last_run_success_bool'] == 'True' and poll_dict['firsttime_run'] == 'False':
            # if the last run was succesful and it isnt the first time polling has been run then it will use the last
            # run datetime for from
            poll_date = datetime.strptime(poll_dict['last_run_datetime'], '%Y-%m-%dT%H:%M:%S.%fZ')
            logging.debug("poll_date={0}".format(poll_date))
            to_str, from_str = api_utils.calculate_from_to(days, poll_date)
        elif poll_dict['last_run_success_bool'] == '':
            # If there is no information on the last successful run time then it will use the last run datetime
            poll_date = datetime.strptime(poll_dict['last_run_datetime'], '%Y-%m-%dT%H:%M:%S.%fZ')
            # this is then passed to calculate the correct datetime
            to_str, from_str = api_utils.calculate_from_to(days, poll_date)
    return to_str, from_str


def prepare_poll(events, temp_exists, from_str):
    poll_al_ids_path = api_conf.poll_alerts_path
    poll_alerts_path = get_file_location(poll_al_ids_path)
    # re-check whether the alert_ids json file exists
    alertids_exists = path.isfile(poll_alerts_path)
    logging.info("Does the alert_ids.json file exist? {0}".format(alertids_exists))

    # Open the poll config to configure polling
    poll_path = api_conf.poll_conf_path
    final_poll_conf_path = get_file_location(poll_path)
    with open(final_poll_conf_path, 'r') as pa_file:
        poll_dict = json.load(pa_file)

    if not alertids_exists:
        # If the alert_ids file does not already exist return unless not first time run or loop
        if poll_dict['firsttime_run'] == "True" and poll_dict['first_loop'] == "True":
            # Set the first loop to false if it is the first time run and the first loop
            logging.info("Polling not previously run")
            # set the first loop to false to determine behaviour on next loop
            poll_dict.update({"first_loop": "False"})

            with open(final_poll_conf_path, 'w+') as pa_file:
                json.dump(poll_dict, pa_file, ensure_ascii=False, indent=2)

            return events
        elif poll_dict['firsttime_run'] == "False" and poll_dict['first_loop'] == "False":
            # send the events to be checked if this isnt the first time that polling has been run and is not the first loop
            logging.info("alert_ids.json file doesnt exist. Polling config shows this is not the first time run"
                         " and is not the first loop")
            # send for checks
            spl_events = alert_file_checks(poll_dict, events, temp_exists, from_str)
            # return the checked events for processing
            return spl_events
        elif poll_dict['first_loop'] == "False":
            # just return the events
            return events
    elif alertids_exists:
        # If the file exists, check existing alerts sent returned and prepared to ensure duplicates are not sent
        logging.info("alert_ids file found, running checks")
        logging.debug("First time run: {0}".format(poll_dict['firsttime_run']))
        logging.debug("First loop: {0}".format(poll_dict['first_loop']))
        # send for checks
        spl_events = alert_file_checks(poll_dict, events, temp_exists, from_str)
        return spl_events


def alert_file_checks(poll_dict, events, temp_exists, from_str):
    poll_path = api_conf.poll_conf_path
    final_poll_conf_path = get_file_location(poll_path)
    # only run checks if the alert_ids file exists
    if poll_dict['firsttime_run'] == "True" and poll_dict['first_loop'] == "True":
        poll_dict.update({"first_loop": "False"})
        with open(final_poll_conf_path, 'w+') as pa_file:
            json.dump(poll_dict, pa_file, ensure_ascii=False, indent=2)
        # if this is the firstime run and first loop, then just return the events
        return events
    elif poll_dict['firsttime_run'] == "True" and poll_dict['first_loop'] == "False":
        logging.debug("First time run is: {0}".format(poll_dict['firsttime_run']))
        # if this is the firstime run then just return the events
        return events
    elif poll_dict['firsttime_run'] == "False" and poll_dict['first_loop'] == "True":
        logging.debug("First time run is: {0} and first loop: {1}".format(poll_dict['firsttime_run'],
                                                                          poll_dict['first_loop']))

        # Prep first loop to get events and calculate datetime.
        from_datetime, process_alerts = first_loop_prep(temp_exists, from_str)

        # Go through existing alerts and delete events older than time frame being gathered
        processed_existing_events = remove_old_alert_ids(from_datetime, process_alerts)

        # Write the necessary events to file
        poll_tf_path = api_conf.poll_temp_path
        poll_temp_path = get_file_location(poll_tf_path)
        with open(poll_temp_path, 'w+', encoding='utf-8') as new_id_file:
            json.dump(processed_existing_events, new_id_file, ensure_ascii=False, indent=2)

        # Reset first loop to ensure events are maintained on next pass
        poll_dict.update({"first_loop": "False"})
        with open(final_poll_conf_path, 'w+') as pa_file:
            json.dump(poll_dict, pa_file, ensure_ascii=False, indent=2)

        logging.info("deleting alert_ids.json")
        poll_al_ids_path = api_conf.poll_alerts_path
        poll_alerts_path = get_file_location(poll_al_ids_path)
        remove(poll_alerts_path)

        # Go through events gathered from Sophos Central and remove entries already sent
        spl_events = process_poll_events(events)

        return spl_events
    elif poll_dict['firsttime_run'] == "False" and poll_dict['first_loop'] == "False":
        logging.info("Continue to process poll events")
        # Go through events gathered from Sophos Central and remove entries already sent
        spl_events = process_poll_events(events)

        return spl_events


def first_loop_prep(temp_exists, from_str):
    # Check whether the temp alert file has been already created from previous run
    if temp_exists:
        # if the file exists clear it down for this new run
        logging.info("temp alerts file exists, clear it down")
        poll_tf_path = api_conf.poll_temp_path
        poll_temp_path = get_file_location(poll_tf_path)
        with open(poll_temp_path, 'w', encoding='utf-8') as reset_temp:
            temp_dict = {}
            json.dump(temp_dict, reset_temp)

    # Open the alert ids and load into json for checking
    poll_al_ids_path = api_conf.poll_alerts_path
    poll_alerts_path = get_file_location(poll_al_ids_path)
    with open(poll_alerts_path, 'r', encoding='utf-8') as existing_alerts:
        # Load the file as json
        process_alerts = json.load(existing_alerts)
        logging.debug(process_alerts)
        logging.info("Length of events from alert_ids.json: {0}".format(len(process_alerts)))

        # Set the correct from string to a datetime and create new list and reset the count
        logging.debug(from_str)
        from_datetime = datetime.strptime(from_str, '%Y-%m-%dT%H:%M:%S.%fZ')
        return from_datetime, process_alerts


def remove_old_alert_ids(from_datetime, process_alerts):
    # Loop through the id_alerts and compare the datetime. If the alert is greater or equal to the
    # from string then keep this alert id and append to new list
    new_ids = {}
    for (event_id, event_value) in process_alerts.items():
        logging.debug("key: {0}, value: {1}".format(event_id, event_value))
        value_date = datetime.strptime(event_value['raised_at'], '%Y-%m-%dT%H:%M:%S.%fZ')
        logging.debug("{0} >= {1}".format(value_date, from_datetime))
        if value_date >= from_datetime:
            id_data = {event_id: {'raised_at': event_value['raised_at'], 'tenant_id': event_value['tenant_id']}}
            new_ids.update(id_data)

    logging.info("Length of events from alert_ids.json after old ones removed: {0}".format(len(new_ids)))

    return new_ids


def process_poll_events(events):
    # Search and remove any ids which have already been sent and build new json
    poll_tf_path = api_conf.poll_temp_path
    poll_temp_path = get_file_location(poll_tf_path)
    logging.info("Begin processing events...")
    with open(poll_temp_path, 'r') as check_alerts:
        temp_alert_ids = json.load(check_alerts, encoding='utf-8')
        new_spl_events = {}

    # check for matching ids and only add missing ones
    for (new_event_id, new_event_value) in events.items():
        if new_event_value['event']['id'] in temp_alert_ids.keys():
            # if the id matches the key then skip
            pass
        else:
            # if there is no match add event to new_spl_events
            new_ev = {new_event_id: new_event_value}
            new_spl_events.update(new_ev)

    logging.info("Length of new_spl_events: {0}".format(len(new_spl_events)))
    logging.debug("new_spl_events type is: {0}".format(type(new_spl_events)))

    # determine length of the new alerts after checking
    if len(new_spl_events) > 0 or len(new_spl_events) != "0":
        # if the number of alerts is not zero then return the events
        return new_spl_events
    else:
        # if the number of events is zero then set the new_spl_events to None so it is dealt with accordingly
        logging.info("No events to send")
        new_spl_events = None
        return new_spl_events


def gen_idalerts(event_data):
    # When an event has been successfully sent to splunk the event data is written to file
    poll_al_ids_path = api_conf.poll_alerts_path
    poll_alerts_path = get_file_location(poll_al_ids_path)
    alertids_exists = path.isfile(poll_alerts_path)
    if alertids_exists:
        # if the alert_ids.json file already exists load the existing information to a variable
        with open(poll_alerts_path, 'r') as alerts_file:
            alert_ids = json.load(alerts_file)

        # update the dictionary with the new event information that was successfully sent to
        alert_ids.update(event_data)
        with open(poll_alerts_path, 'w', encoding='utf-8') as id_file:
            json.dump(alert_ids, id_file, ensure_ascii=False, indent=2)

    else:
        # if the file doesnt yet exist then create it with the event data
        with open(poll_alerts_path, 'w', encoding='utf-8') as alerts_file:
            json.dump(event_data, alerts_file, ensure_ascii=False, indent=2)


def finalise_polling(poll, to_str):
    poll_path = api_conf.poll_conf_path
    final_poll_conf_path = get_file_location(poll_path)
    poll_al_ids_path = api_conf.poll_alerts_path
    poll_alerts_path = get_file_location(poll_al_ids_path)
    poll_tf_path = api_conf.poll_temp_path
    poll_temp_path = get_file_location(poll_tf_path)
    # once there are no further events to process for all of the tenants need to finalise the polling information
    # ready for the next run
    exists = path.isfile(final_poll_conf_path)
    temp_exists = path.isfile(poll_temp_path)
    main_exists = path.isfile(poll_alerts_path)

    def get_old_alerts():
        # gather the old sent alerts from the temp file if any
        if temp_exists:
            with open(poll_temp_path, 'r') as check_alerts:
                temp_alert_ids = json.load(check_alerts, encoding='utf-8')
            return temp_alert_ids
        else:
            temp_alert_ids = None
            return temp_alert_ids

    def get_new_alerts():
        # gather the new events that have been sent if any
        if main_exists:
            with open(poll_alerts_path, 'r') as new_alerts:
                alerts = json.load(new_alerts, encoding='utf-8')
            return alerts
        else:
            alerts = None
            return alerts

    if temp_exists:
        # combine the new and old events into one file if temp exists
        logging.info("Combining old and new events into new alert_ids.json file")
        # gather old events
        old_alerts = get_old_alerts()
        logging.debug("old_alerts type: {0}".format(type(old_alerts)))
        # gather new events
        new_alerts = get_new_alerts()
        logging.debug("new_alerts type: {0}".format(type(new_alerts)))

        if new_alerts is None:
            # if there are no new alerts then just dump the old alerts to alert_ids.json
            with open(poll_alerts_path, 'w', encoding='utf-8') as maintain_alerts:
                json.dump(old_alerts, maintain_alerts, ensure_ascii=False, indent=2)
        else:
            # if there are new alerts then go through old alerts and add them to the new events in alert_ids.json
            for (old_id, old_id_value) in old_alerts.items():
                old_ev = {old_id: old_id_value}
                new_alerts.update(old_ev)

            with open(poll_alerts_path, 'w', encoding='utf-8') as combined_alerts:
                json.dump(new_alerts, combined_alerts, ensure_ascii=False, indent=2)
    else:
        # if the temp file doesnt exist then skip this part
        pass

    if exists and poll:
        # if the polling config exists and the polling is set then update the config according to the result of the pass
        logging.info("Update poll_config information")
        with open(final_poll_conf_path, 'r') as pa_file:
            poll_dict = json.load(pa_file)

            logging.debug(poll_dict)

            if poll_dict['failures_seen'] == "False":
                # if there were no failures seen then update the relevant entries for success
                logging.info("No failures seen")
                poll_dict['last_run_success_bool'] = 'True'
                poll_dict['last_run_success_datetime'] = to_str
                poll_dict['first_loop'] = 'True'
                poll_dict['firsttime_run'] = 'False'
                poll_dict['last_run_datetime'] = to_str
            elif poll_dict['failures_seen'] == "True":
                # if there were failures seen then update the relevant entries for failure so we capture them in the
                # next pass
                logging.info("Failures seen, setting config")
                poll_dict['last_run_success_bool'] = 'False'
                poll_dict['first_loop'] = 'True'
                poll_dict['firsttime_run'] = 'False'

        # write the changes to the polling config
        with open(final_poll_conf_path, 'w') as pa_file:
            json.dump(poll_dict, pa_file, indent=2)
    else:
        pass


def get_file_location(process_path):
    dir_name = path.dirname(__file__)
    final_path = "{0}{1}".format(dir_name, process_path)
    return final_path



import requests
import logging


def gen_tenant_headers(headers, whoami_id, whoami_type, header_type):
    # Check whether we have received a valid uuid
    if whoami_id is not None:
        logging.info("Applying the uuid to the header")
        # Apply the uuid to the variable and add it to the correct header
        headers[header_type] = whoami_id
        if whoami_type == "tenant":
            # the type is tenant so just return the data to gather the events
            logging.info("uuid is tenant only. Applying to the tenant headers")
            logging.info("Skipping collecting further information on the tenant ids")
            tenant_headers = {"{0}".format(header_type): "{0}".format(whoami_id)}
            return tenant_headers
        else:
            # return that the type is not tenant so the tenant information can be gathered
            tenant_headers = None
            return tenant_headers
    else:
        # Return the response code and error information and exit the script
        logging.error("Response Code: {0} Error Code: {1}".format(whoami_type, header_type))
        exit(1)


# This function will get the page total and pass the data to get_page function to gather the tenant information
def get_tenant_info(headers, tenant_url, sophos_access_token):
    # Get the total number of pages
    page_total_url = "{0}{1}".format(tenant_url, "?pageTotal=true")
    res_tenant = requests.get(page_total_url, headers=headers)
    res_tenant_code = res_tenant.status_code
    tenant_data = res_tenant.json()

    # Check the response and if good pass to get_page
    if res_tenant_code == 200:
        logging.info("Total tenant page information gathered")
        tenant_page_total = tenant_data['pages']['total']
        tenant_info = dict()
        # Call get_page function
        get_next_page(tenant_url, headers, tenant_page_total, sophos_access_token, tenant_info)
        return tenant_info
    else:
        # If the connection is not successful then pass back errors
        logging.error("Failed to get tenant information")
        res_tenant_error_code = tenant_data['error']
        return None, res_tenant_code, res_tenant_error_code


def get_next_page(tenant_url, headers, tenant_page_total, sophos_access_token, tenant_info):
    # This function constructs the url to gather pages of tenant information
    next_page = 1
    paged_url = "{0}{1}{2}".format(tenant_url, "?page=", next_page)
    # while page_url is not null it will calculate the next page and append tenant info to tenant_list
    while paged_url:
        res_tenant = requests.get(paged_url, headers=headers)
        tenant_data = res_tenant.json()
        # from the tenant data construct the headers to connect to the correct api url
        for item in tenant_data['items']:
            tenant_id = "{0}".format(item['id'])
            tenant_headers = {"X-Tenant-ID": tenant_id, "Authorization": "Bearer {0}".format(sophos_access_token),
                              "Accept": "application/json"}
            tenant_item = {item['id']: {"name": item['name'], "headers": tenant_headers, "page_url": item['apiHost']}}
            tenant_info.update(tenant_item)

        next_page += 1
        # checks if the next_page value is less than or equal to total pages. If so create next url for pass
        if next_page <= tenant_page_total:
            paged_url = "{0}{1}{2}".format(tenant_url, "?page=", next_page)
        else:
            paged_url = False


def type_tenant(tenant_headers, whoami_id, tenant_url, sophos_access_token):
    # Creates a tenant object if the whoami is solely a tenant object. This will not construct headers for Partner or
    # Organisation
    tenant_headers["Authorization"] = "Bearer {0}".format(sophos_access_token)
    tenant_headers["Accept"] = "application/json"
    tenant_info = {whoami_id: {"name": "tenant", "headers": tenant_headers, "page_url": tenant_url}}
    return tenant_info



import logging
import requests
from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from sophos_central_api_connector import sophos_central_api_connector_utils as api_utils


def get_data(tenant_url_data, page_size, tenant_id, api):
    # start the process to get the information from the api, this is standard function and is based on the url
    # the calculation is done through building the urls in sophos_central_api_connector_utils.py
    logging.debug("get data tenant info orig_url: {0}".format(tenant_url_data[tenant_id]['orig_url']))
    pageurl = tenant_url_data[tenant_id]['pageurl']
    orig_url = tenant_url_data[tenant_id]['orig_url']
    headers = tenant_url_data[tenant_id]['headers']
    json_items = dict()

    # gather page total information
    if api != "common":
        pagetotal_url = "{0}?pageSize={1}&pageTotal=true".format(orig_url, page_size)
        logging.debug("page total url: {0}".format(pagetotal_url))
        pg_total = get_page_totals(pagetotal_url, headers)
    else:
        logging.debug("pg_total = None")
        pg_total = None

    # get the first page of data from central api
    ep_data = get_page(pageurl, headers)

    # attribute the retrieved data from the json to item and page data variables
    if not ep_data or ep_data.get('error'):
        ep_item_data = None
    else:
        ep_item_data = ep_data['items']
        ep_page_data = ep_data.get('pages')

        if ep_page_data.get('current'):
            page_no = ep_page_data['current']
        else:
            page_no = None

    if not ep_item_data:
        # Checks if any events have been obtained. If not then sets the next key to false
        logging.info("No data returned for this Tenant ID: '{0}'".format(tenant_id))
        next_key = False
    elif len(ep_item_data) == 0:
        # Checks if any events have been obtained. If not then sets the next key to false
        logging.info("There are no pages to process for this Tenant ID: '{0}'".format(tenant_id))
        next_key = False
    elif page_no == pg_total:
        # Checks if any events have been obtained. If not then sets the next key to false
        logging.info("There are no further pages to process for this Tenant ID: '{0}'".format(tenant_id))
        next_key = False
    else:
        # Does further checks if there is data. to verify whether we need to send for the next page of events
        if 'nextKey' in ep_page_data.keys():
            # Only if the nextkey is present will we send further requests for data
            logging.debug("Next page to process")
            next_key = True
        elif page_no is not None and page_no < pg_total:
            # If the api does not contain a nextkey check whether it has more pages to process
            logging.debug("Current page is lower than the total number of pages. Continue processing")
            next_key = True
        else:
            # No nextkey is present we set next_key to false
            logging.debug("There is no 'next' key present")
            next_key = False

    # set the event count to 0
    event_count = 0
    # build the json data for the items retrieved
    if not ep_item_data:
        pass
    else:
        json_items, event_count = api_utils.build_json_data(ep_item_data, json_items, event_count)
        logging.debug("Length of json: {0}".format(len(json_items)))

    # Check if there is a next page to process
    while next_key:
        # this will only trigger if the next_key variable is True
        # send the relevant information to obtain the next page
        next_key, pageurl = process_next_page(ep_page_data, page_size, orig_url, page_no, pg_total)
        if pageurl:
            # will only run if the pageurl is not None
            ep_data = get_page(pageurl, headers)
            ep_item_data = ep_data['items']
            ep_page_data = ep_data['pages']
            if 'current' in ep_page_data.keys():
                page_no = ep_data['pages']['current']
            else:
                page_no = None
            # add the events obtained to the json data dictionary
            json_items, event_count = api_utils.build_json_data(ep_item_data, json_items, event_count)
            logging.debug("Length of json: {0}".format(len(json_items)))
        else:
            logging.info("No further pages to process")

    # Once the next_key is false return the json_items retrieved
    logging.debug("JSON Data from get data: {0}".format(json_items))
    return json_items


def get_page(pageurl, headers):
    # attempt to get data from sophos central api
    logging.debug("get_page url passed: {0}".format(pageurl))
    res_sess = requests.session()
    retries = Retry(total=10, backoff_factor=0.1, status_forcelist=[500, 502, 503, 504, 429])
    res_sess.mount('https://', HTTPAdapter(max_retries=retries))
    try:
        logging.debug("Attempting to get page: {0}".format(pageurl))
        ep_res = res_sess.get(pageurl, headers=headers)
        ep_res.raise_for_status()
    except requests.exceptions.HTTPError as err_http:
        if err_http.response == 403:
            pass
        else:
            error_content = ep_res.content.decode()
            return err_http.strerror
    except requests.exceptions.ConnectionError as conn_err:
        logging.error(conn_err)
        return conn_err.strerror
    else:
        # success response pass back data to build json
        if ep_res:
            ep_data = ep_res.json()
        else:
            ep_data = None

        ep_res.close()
        return ep_data


def process_next_page(ep_page_data, page_size, orig_url, page_no, pg_total):
    # if a next_key has been provided the process next page function is called to construct the url based on api type
    if 'nextKey' in ep_page_data.keys():
        # builds new url is next_key present
        ep_nextkey = ep_page_data['nextKey']
        logging.info(
            "There is another page to process, resending request with new page from key: {0}".format(ep_nextkey))
        pageurl = "{0}?pageFromKey={1}&pageSize={2}".format(orig_url, ep_nextkey, page_size)
        next_key = True
        return next_key, pageurl
    elif page_no is not None and page_no < pg_total:
        # builds new url if page no. hasn't reached page total
        page_no += 1
        pageurl = "{0}?page={1}&pageSize={2}".format(orig_url, page_no, page_size)
        logging.info("There is another page to process, resending request with new page value: {0}".format(page_no))
        next_key = True
        return next_key, pageurl
    else:
        # if there is no next_key then returns the next page as false to break out of while loop
        next_key = False
        pageurl = None
        return next_key, pageurl


def get_page_totals(url, headers):
    try:
        pagetotal_res = requests.get(url, headers=headers)
        pagetotal_res.raise_for_status()
    except requests.exceptions.HTTPError as http_err:
        if pagetotal_res.status_code == 403:
            pass
        else:
            logging.error(http_err)
            pg_total = None
            return pg_total
    else:
        pg_info = pagetotal_res.json()
        pg_data = pg_info.get('pages')
        if pg_data:
            pg_total = pg_data['total']
            logging.debug("page total data: {0}".format(pg_data))
            return pg_total
        else:
            pg_total = None
            return pg_total



import setuptools
from os import path

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(name='sophos_central_api_connector',
                 version='0.1.4',
                 description='Sophos Central API Connector',
                 author='Mark Lanczak-Faulds',
                 author_email='mark.lanczak-faulds@sophos.com',
                 long_description=long_description,
                 long_description_content_type='text/markdown',
                 classifiers=[
                     "Programming Language :: Python :: 3",
                     "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
                     "Operating System :: OS Independent",
                 ],
                 url="https://github.com/sophos-cybersecurity/sophos-central-api-connector",
                 setup_requires=["setuptools_scm"],
                 packages=['sophos_central_api_connector', 'sophos_central_api_connector.config'],
                 package_data={'sophos_central_api_connector': ['config/*']},
                 python_requires='>=3.6',
                 install_requires=['boto3>=1.17.21',
                                   'botocore>=1.20.21',
                                   'certifi>=2020.12.5',
                                   'chardet>=4.0.0',
                                   'configparser>=5.0.2',
                                   'docutils>=0.16',
                                   'idna>=3.1',
                                   'intelix>=0.1.2',
                                   'jmespath>=0.10.0',
                                   'python-dateutil>=2.8.1',
                                   'requests>=2.25.1',
                                   's3transfer>=0.3.4',
                                   'six>=1.15.0',
                                   'urllib3>=1.26.3'],
                 include_package_data=True)



import logging
import json
import requests
from urllib3.util.retry import Retry
from time import sleep
from sophos_central_api_connector import sophos_central_api_get_data as get_api

api = "live-discover"
page_size = "250"


def live_discover(tenant_data, input_type, input_value, search_filter, variables):
    def gather_ld():
        # build the payload to post
        logging.info("Building payload information for saved search")
        payload_dict = build_json_payload(input_type, input_value, query_det, search_filter, variables)

        # post run
        logging.info("Start posting saved queries")
        post_dict = post_ld_run(tenant_data, payload_dict)
        logging.debug("post dict: {0}".format(post_dict))

        # run checker
        logging.info("Start checking query status")
        run_info = multi_query_run_checker(tenant_data, post_dict)
        logging.debug("Run info: {0}".format(run_info))

        # get endpoint data
        logging.info("Gather endpoint information")
        ep_info = get_ld_endpoints(tenant_data, run_info)
        logging.debug("Endpoint information: {0}".format(ep_info))

        # get run results
        logging.info("Gather results based on run ids")
        size = 1000
        result_dict = get_run_results(tenant_data, run_info, size)
        logging.debug("Result data: {0}".format(result_dict))

        return run_info, ep_info, result_dict

    if input_type == "saved":
        # Get saved query details
        logging.info("Getting saved search details")
        query_det = get_queries(tenant_data, True, input_value)
        if not variables:
            for s_id, val in query_det.items():
                for t_id, s_val in val.items():
                    if s_val['template_variables']:
                        variables = s_val['template_variables']
                    else:
                        pass
        else:
            pass

        ld_data, ep_data, result_data = gather_ld()
        return ld_data, ep_data, result_data

    elif input_type == "adhoc":
        logging.info("Coming soon")
        # todo: check the size of the adhoc query
        exit(1)
    elif input_type == "list":
        query_list = get_queries(tenant_data, False, None)
        return query_list


def data_lake():
    logging.info("Coming soon")
    exit(1)


def get_queries(tenant_url_data, query_find, query_det):
    def find_query(query_dict, query_info, tenant, fdict):
        logging.info("Looking for saved query in dictionary")

        for num, item in query_dict.items():
            if item.get('name') == query_info:
                q_id = item['id']
                logging.info("Found saved query: '{0}' for tenant: {1}".format(query_det, tenant))
                logging.info("Search ID: {0}".format(q_id))
                if item['variables']:
                    if fdict.get(tenant):
                        fdict.update({tenant: {q_id: {"name": item['name'], "description": item['description'],
                                                      "template_variables": item['variables']}}})
                    else:
                        fdict[tenant] = {q_id: {"name": item['name'], "description": item['description'],
                                                "template_variables": item['variables']}}
                else:
                    if fdict.get(tenant):
                        fdict[tenant] = {q_id: {"name": item['name'], "description": item['description'],
                                                "template_variables": None}}
                    else:
                        fdict.update({tenant: {q_id: {"name": item['name'], "description": item['description'],
                                                      "template_variables": None}}})
                logging.debug("Query details: {0}".format(fdict))
            else:
                pass

        return fdict

    def list_queries(query_list, tenant, qdict):
        for key, val in query_list.items():
            q_id = val.get('id')
            q_name = val.get('name')
            q_desc = val.get('description')
            q_var = val.get('variables')
            qdict[q_id] = {"name": q_name, "description": q_desc, "variables": q_var, "tenant_id": tenant}

        return qdict

    # Gather the queries for the tenants
    new_dict = dict()
    logging.info("Attempting to get Live Discover queries")

    for ten_id, ten_item in tenant_url_data.items():
        # Pass the ten_url_data and gather the queries
        tenant_id = ten_id
        # get data information for the tenant in the loop
        json_data = get_api.get_data(tenant_url_data, page_size, tenant_id, api)
        # Check if query name passed is valid and present
        if query_find:
            new_dict = find_query(json_data, query_det, tenant_id, new_dict)
        else:
            # Print list of available queries
            new_dict = list_queries(json_data, tenant_id, new_dict)

    return new_dict


def build_json_payload(input_type, input_value, query_data, filters, variables):
    # Open find query json if available
    rebuild_dict = dict()

    for k, v in query_data.items():
        for s_id, s_det in v.items():
            q_name = s_det['name']
            q_desc = s_det['description']
            if variables:
                try:
                    if input_type == "saved":
                        con_json = {"matchEndpoints": {"filters": [filters]}, "savedQuery": {"queryId": s_id},
                                    "variables": variables}
                    elif input_type == "adhoc":
                        con_json = {"matchEndpoints": {"filters": [filters]}, "adHocQuery": {"template": input_value},
                                    "variables": variables}
                    else:
                        logging.error("Input type is not valid, please review variables passed")
                        exit(1)

                    json.dumps(con_json)
                except ValueError as err:
                    logging.error("JSON has failed validation test")
                    logging.error(con_json)
                    raise err
                else:
                    logging.info("JSON has been validated successfully")
                    logging.debug("JSON Passed: {0}".format(con_json))
                    logging.info("Adding payload to query dict")
                    rebuild_dict[k] = {s_id: {"name": q_name, "description": q_desc, "payload": con_json}}
            elif not variables:
                try:
                    if input_type == "saved":
                        con_json = {"matchEndpoints": {"filters": [filters]}, "savedQuery": {"queryId": s_id}}
                    elif input_type == "adhoc":
                        con_json = {"matchEndpoints": {"filters": [filters]}, "adHocQuery": {"template": input_value}}
                    else:
                        logging.error("Input type is not valid, please review variables passed")
                        exit(1)

                    json.dumps(con_json)
                except ValueError as err:
                    logging.error("JSON has failed validation test")
                    logging.error(con_json)
                    raise err
                else:
                    logging.info("JSON has been validated successfully")
                    logging.info("Adding payload to query dict")
                    rebuild_dict[k] = {s_id: {"name": q_name, "description": q_desc, "payload": con_json}}
            else:
                pass

    return rebuild_dict


def post_ld_run(tenant_url_data, ld_dict):
    def post_query(query_data, url, headers):
        q_payload = query_data['payload']
        try:
            logging.info("Posting query")
            post_res = requests.post(url, headers=headers, json=q_payload)
            post_res.raise_for_status()
        except requests.exceptions.HTTPError:  # as http_err:
            if post_res.status_code == 404:
                logging.error("Item not found. Please verify filter for this query id")
                p_data = post_res.json()
                logging.debug(json.dumps(p_data, indent=2))
                return p_data
                # raise http_err
            elif post_res.status_code == 400:
                logging.error(post_res.text)
                p_data = post_res.json()
                logging.debug(json.dumps(p_data, indent=2))
                return p_data
                # raise http_err
            else:
                logging.error("Error whilst posting search query, response: {0}".format(post_res.status_code))
                p_data = post_res.json()
                logging.debug(json.dumps(p_data, indent=2))
                return p_data
                # raise http_err
        else:
            logging.info("Post successful, making note of post id")
            p_data = post_res.json()
            logging.debug(json.dumps(p_data, indent=2))
            return p_data

    # Prep for posting queries to live discover
    post_url = "/runs"
    pq_dict = dict()

    for key, value in tenant_url_data.items():
        pq_url = "{0}{1}".format(value['orig_url'], post_url)
        tenant = key
        p_headers = value['headers']
        p_headers['Content-Type'] = "application/json"
        for k, v in ld_dict.items():
            for q_key, q_val in v.items():
                if tenant == k:
                    ten_id = k
                    logging.info("Search found for tenant: {0}, search name: {1}".format(ten_id, q_val['name']))
                    logging.info("Search ID: {0}".format(q_key))
                    post_data = post_query(q_val, pq_url, p_headers)
                    if post_data.get('id'):
                        pq_id = post_data['id']
                        pq_created = post_data['createdAt']
                        pq_status = post_data['status']
                        ep_count = post_data['endpointCounts']['total']
                        pq_dict[k] = {q_key: {"name": q_val['name'], "description": q_val['description'],
                                              "payload": q_val['payload'],
                                              "post_data": {"pq_id": pq_id, "createdAt": pq_created,
                                                            "status": pq_status,
                                                            "ep_count": ep_count}}}
                        # pq_dict.update(pq_data)
                    elif post_data.get('error'):
                        pq_err = post_data['error']
                        pq_errmsg = post_data['message']
                        r_id = post_data['requestId']
                        pq_dict[k] = {q_key: {"name": q_val['name'], "description": q_val['description'],
                                              "payload": q_val['payload'],
                                              "post_data": {"r_id": r_id, "error": pq_err, "error_message": pq_errmsg}}}
                else:
                    logging.debug("No searches found in list for tenant: {0}".format(tenant))

    return pq_dict


def multi_query_run_checker(tenant_url_data, ld_dict):
    def loop_check():
        logging.debug("loop check")
        run_check = True
        for t_key, pval in ld_dict.items():
            for sid, val in pval.items():
                if val.get('post_data').get('pq_id'):
                    new_status = val.get('post_data').get('status')
                    t_remaining = val.get('post_data').get('time_remaining')
                    if new_status != "finished":
                        if t_remaining == 0:
                            logging.error("Max query duration reached. Query '{0}' will be terminated".format(sid))
                            val['post_data']['terminated'] = True
                            pass
                        else:
                            val['post_data']['terminated'] = False
                            logging.info("Time remaining before query, {0}: {1}, is automatically terminated: {2} "
                                         "seconds".format(val['name'], sid, t_remaining))
                            run_check = False

        return run_check

    # Check the run status of the queries being run
    q_fin = False
    logging.info("Checking status of run(s). Check every 10s until all have finished")
    while not q_fin:
        # For loop to go through dict and check all run ids
        for ten_key, vals in ld_dict.items():
            for s_id, s_vals in vals.items():
                if s_vals.get('post_data').get('pq_id'):
                    run_id = s_vals['post_data']['pq_id']
                    run_status = get_ld_run_status(tenant_url_data, ten_key, run_id)
                    if run_status:
                        status = run_status.get('status')
                        logging.debug("run status: {0}".format(status))
                        s_vals['post_data']['status'] = status
                        s_vals['post_data']['time_remaining'] = run_status['timeRemainingInSeconds']
                        s_vals['post_data']['finishedAt'] = run_status.get('finishedAt', None)
                        s_vals['post_data']['statuses'] = run_status['endpointCounts']['statuses']
                    else:
                        logging.info("No run status information for search id: {0}".format(run_id))

        q_fin = loop_check()
        if not q_fin:
            logging.debug("Sleep for 10s")
            sleep(10)
    else:
        logging.info("All query runs have finished")
        return ld_dict


def get_ld_run_status(tenant_url_data, tenant, run_id):
    url_ext = "/runs/{0}".format(run_id)

    for key, value in tenant_url_data.items():
        if key == tenant:
            url = "{0}{1}".format(value['orig_url'], url_ext)
            headers = value['headers']
            headers['Content-Type'] = "application/json"
            logging.info("Check run id: {0}".format(run_id))
            run_status = get_api.get_page(url, headers)
        else:
            pass

    return run_status


def get_ld_endpoints(tenant_url_data, run_data):
    def get_runid_eps():
        for key, value in tenant_url_data.items():
            if key == tenant:
                url = "{0}{1}".format(value['orig_url'], url_ext)
                pagetotal_url = "{0}?pageSize={1}&pageTotal=true".format(url, page_size)
                headers = value['headers']
                headers['Content-Type'] = "application/json"
                ten_data = dict()
                ten_data[key] = {'filename': value['filename'], 'url': value['url'], 'orig_url': url,
                                 'pageurl': pagetotal_url, 'name': value['name'], 'headers': headers}
                # Get the page totals
                logging.debug("get_runid_eps loop ten info orig_url: {0}".format(ten_data[key]['orig_url']))
                page_data = get_api.get_data(ten_data, page_size, tenant, api)
                logging.debug("get_runid_eps loop, page data: {0}".format(page_data))
                return page_data

    ld_ep_dict = dict()

    for t_key, v in run_data.items():
        for s_id, r_value in v.items():
            if r_value.get('post_data').get('pq_id'):
                run_id = r_value['post_data']['pq_id']
                tenant = t_key
                url_ext = "/runs/{0}/endpoints".format(run_id)
                pg_info = get_runid_eps()
                ld_ep_dict[run_id] = pg_info

    return ld_ep_dict


def get_run_results(tenant_url_data, run_data, size):
    def get_results():
        for key, value in tenant_url_data.items():
            if key == tenant:
                url = "{0}{1}".format(value['orig_url'], url_ext)
                pagetotal_url = "{0}?pageSize={1}&pageTotal=true".format(url, size)
                headers = value['headers']
                headers['Content-Type'] = "application/json"
                ten_data = dict()
                ten_data[key] = {'filename': value['filename'], 'url': value['url'], 'orig_url': url,
                                 'pageurl': pagetotal_url, 'name': value['name'], 'headers': headers}
                # Get the page totals
                logging.info("Gather page data for results")
                page_data = get_api.get_data(ten_data, size, tenant, api)
                return page_data

    q_res_dict = dict()

    # For loop to go through the dict and check all run ids
    for ten_key, val in run_data.items():
        for s_id, s_vals in val.items():
            if s_vals.get('post_data').get('pq_id'):
                run_id = s_vals['post_data']['pq_id']
                url_ext = "/runs/{0}/results".format(run_id)
                success_withdata = s_vals['post_data']['statuses']['finished']['succeeded']['withData']
                terminated = s_vals.get('post_data').get('terminated')
                logging.info("Checking whether any endpoints completed with data")
                if success_withdata > 0:
                    logging.info("Some endpoints have returned data")
                    logging.info("Waiting 2 minutes to start gathering results")
                    tenant = ten_key
                    sleep(120)
                    res_data = get_results()
                    if q_res_dict.get(run_id):
                        logging.info("Results gathered")
                        q_res_dict.update({run_id: {res_data}})
                    else:
                        logging.info("Results gathered")
                        q_res_dict[run_id] = res_data
                elif success_withdata == 0 and terminated:
                    logging.info("This query was terminated, with no results. Skipping")
                else:
                    logging.info("No results have been returned for this run id: '{0}'".format(run_id))
                    if q_res_dict.get(run_id):
                        q_res_dict.update({run_id: {"tenant": ten_key, "results": "No EPs completed with data"}})
                    else:
                        q_res_dict[run_id] = {"tenant": ten_key, "results": "No EPs completed with data"}
            else:
                logging.info("No results have been returned")
                if q_res_dict.get(run_id):
                    r_id = s_vals['post_data']['r_id']
                    q_res_dict.update({r_id: {"tenant": ten_key, "results": "No EPs completed with data"}})
                else:
                    q_res_dict[r_id] = {"tenant": ten_key, "results": "No EPs completed with data"}

    return q_res_dict


def build_run_report():
    logging.info("Coming soon")
    exit(1)
    # todo: Based on the results build a report


def get_misp_attributes(misp_url, search_type, search_val, misp_tok, wildcard):
    # create body to search for the attributes
    if search_type == "eventid":
        http_body = {"returnFormat": "json", "eventid": "{0}".format(search_val), "type": {"OR": ["ip-src", "ip-dst",
                                                                                                  "md5", "sha256",
                                                                                                  "filename", "domain",
                                                                                                  "uri", "hostname"]}}
    elif search_type == "tag":
        http_body = {"returnFormat": "json", "tags": "{0}".format(search_val), "type": {"OR": ["ip-src", "ip-dst",
                                                                                               "md5", "sha256",
                                                                                               "filename", "domain",
                                                                                               "uri", "hostname"]}}
    else:
        logging.error("Search type not yet set. Update function: 'get_misp_attributes' with new type")
        return None

    misp_headers = dict()
    misp_headers["Authorization"] = "{0}".format(misp_tok)
    misp_headers["Accept"] = "application/json"
    misp_headers["Content-Type"] = "application/json"

    res_sess = requests.session()
    retries = Retry(total=10, backoff_factor=0.1, status_forcelist=[500, 502, 503, 504, 429])
    res_sess.mount('https://', requests.adapters.HTTPAdapter(max_retries=retries))

    try:
        misp_res = res_sess.post(misp_url, headers=misp_headers, json=http_body)
        misp_res.raise_for_status()
    except requests.exceptions.HTTPError as http_err:
        logging.error(http_err)
        exit(1)
    finally:
        attr_list = list()
        attr_dict = dict()
        json_res = misp_res.json()
        json_resp = json_res['response']
        json_att = json_resp['Attribute']
        for item in json_att:
            if item['type'] == "ip-src" or item['type'] == "ip-dst":
                ind_type = "ip"
            elif item['type'] == "uri":
                ind_type = "url"
            elif item['type'] == "hostname":
                ind_type = "domain"
            else:
                ind_type = item['type']

            if wildcard:
                attr_list.append(
                    {"indicator_type": ind_type, "misp_type": item['type'], "data": "%{0}%".format(item['value'])})
            else:
                attr_list.append(
                    {"indicator_type": ind_type, "misp_type": item['type'], "data": "{0}".format(item['value'])})

        attr_dict['ioc_data'] = attr_list
        attrib = json.dumps(attr_dict)
        res_sess.close()

        return attrib



# Use this code snippet in your app.
# If you need more information about configurations or implementing the sample code, visit the AWS docs:   
# https://aws.amazon.com/developers/getting-started/python/

import json
import boto3
import base64
import logging
from botocore.exceptions import ClientError


def get_secret(secret_name, region_name):
    # Create a Secrets Manager client
    session = boto3.session.Session()
    client = session.client(
        service_name='secretsmanager',
        region_name=region_name
    )

    # In this sample we only handle the specific exceptions for the 'GetSecretValue' API.
    # See https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
    # We rethrow the exception by default.

    try:
        get_secret_value_response = client.get_secret_value(
            SecretId=secret_name
        )
    except ClientError as e:
        if e.response['Error']['Code'] == 'DecryptionFailureException':
            # Secrets Manager can't decrypt the protected secret text using the provided KMS key.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InternalServiceErrorException':
            # An error occurred on the server side.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidParameterException':
            # You provided an invalid value for a parameter.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'InvalidRequestException':
            # You provided a parameter value that is not valid for the current state of the resource.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'ResourceNotFoundException':
            # We can't find the resource that you asked for.
            # Deal with the exception here, and/or rethrow at your discretion.
            raise e
        elif e.response['Error']['Code'] == 'ExpiredTokenException':
            # If you are using a SAML to AWS STS Keys or another token.
            # This error is raised to advise the token is expired
            raise e
    else:
        # Decrypts secret using the associated KMS CMK.
        # Depending on whether the secret is a string or binary, one of these fields will be populated.
        if 'SecretString' in get_secret_value_response:
            logging.info("Got some aws secret info!")
            secret = json.loads(get_secret_value_response['SecretString'])
            return secret
        else:
            logging.info("Got some aws secret info!")
            secret = base64.b64decode(get_secret_value_response['SecretBinary'])
            return secret



import logging
import requests
import shutil
from requests.utils import requote_uri
from datetime import datetime, timedelta
from time import sleep
from os import path
from sophos_central_api_connector.config import sophos_central_api_config as api_conf

# static global entries for the urls of the apis for Sophos Central
endpoint_url = api_conf.endpoints_uri
alerts_url = api_conf.alerts_uri
settings_url = api_conf.settings_uri
livediscover_url = api_conf.livediscover_uri


def gather_category_data(tenant_info):
    # create url for the category information
    logging.info("Gathering categorization data")
    url_values = tenant_info.values()
    url_iter = iter(url_values)
    cat_url = next(url_iter)
    pageurl = "{0}{1}{2}".format(cat_url["page_url"], settings_url, "/web-control/categories")
    headers = dict(cat_url['headers'])

    try:
        cat_res = requests.get(pageurl, headers=headers)
    except requests.exceptions.HTTPError as err_http:
        logging.error("HTTP Error:", err_http)
        logging.info(err_http)
    # if rate limit is hit then attempt to restrict
    if cat_res.status_code == 429:
        error = True
        logging.info("Error {0}: Backing off!".format(cat_res.status_code))
        while error:
            try:
                ep_res = requests.get(pageurl, headers=headers)
            except requests.exceptions.HTTPError as err_http:
                logging.error("HTTP Error:", err_http)
                logging.info(err_http)
                if cat_res.status_code == 200:
                    ep_data = cat_res.json()
                    return ep_data
                elif cat_res.status_code == 429:
                    logging.error("Still hitting the rate limit!")
                    sleep(delay_time)
                    delay_time = delay_time * 2
                    logging.info("Delayed by: {0}".format(delay_time))
                elif cat_res.status_code != 200:
                    logging.error("Response code is not 200")
                    raise Exception('API response: {0}'.format(cat_res.status_code))
    elif cat_res.status_code == 200:
        # success response pass back data to build json
        logging.debug(cat_res.headers)
        ep_data = cat_res.json()
        return ep_data
    else:
        # details on the response code and error
        logging.error("Response Code: {0}".format(cat_res.status_code))
        logging.error("Error Details: {0}".format(cat_res.content))


def generate_tenant_urls(tenant_info, page_size, api, from_str, to_str):
    # dependent on the api type passed this will generate the appropriate headers
    logging.info("Generating tenant URL data")
    # create a dictionary ready to apply the headers for each of the tenants
    tenant_url_data = dict()
    if api == "endpoint":
        # api for endpoint has been passed. For loop to generate the headers for each of the tenant ids
        for ten_id, ten_item in tenant_info.items():
            tenant_url_data[ten_id] = {"filename": "{0}{1}{2}{3}".format(ten_item["name"], "_", ten_id, ".json"),
                                       "orig_url": "{0}{1}".format(ten_item["page_url"], endpoint_url),
                                       "pageurl": "{0}{1}?pageSize={2}".format(ten_item['page_url'], endpoint_url,
                                                                               page_size),
                                       "headers": ten_item["headers"]}
        return tenant_url_data
    elif api == "common":
        # api for common has been passed. For loop to generate the headers for each of the tenant ids
        for ten_id, ten_item in tenant_info.items():
            # a decoded url is constructed from the variables
            decoded_url = str("{0}{1}?from={2}&to={3}&pageSize={4}".format(ten_item["page_url"],
                                                                           alerts_url, from_str, to_str, page_size))
            # the decoded urls are encoded so they are valid urls to be passed
            pageurl = requote_uri(decoded_url)
            tenant_url_data[ten_id] = {"filename": "{0}{1}{2}{3}".format(ten_item["name"], "_", ten_id, ".json"),
                                       "url": "{0}".format(ten_item["page_url"]), "orig_url": pageurl, "pageurl": pageurl,
                                       "headers": ten_item["headers"]}
        return tenant_url_data
    elif api == "local-sites":
        # api for local sites has been passed. For loop to generate the headers for each of the tenant ids
        for ten_id, ten_item in tenant_info.items():
            tenant_url_data[ten_id] = {"filename": "{0}{1}{2}{3}".format(ten_item["name"], "_", ten_id, ".json"),
                                       "orig_url": "{0}{1}{2}".format(ten_item["page_url"], settings_url,
                                                                      "/web-control/local-sites"),
                                       "pageurl": "{0}{1}{2}?pageSize={3}".format(ten_item['page_url'], settings_url,
                                                                                  "/web-control/local-sites",
                                                                                  page_size),
                                       "headers": ten_item["headers"]}
        return tenant_url_data
    elif api == "live-discover":
        # api for live discover has been passed. For loop to generate the headers for each of the tenant ids
        for ten_id, ten_item in tenant_info.items():
            tenant_url_data[ten_id] = {"filename": "{0}{1}{2}{3}".format(ten_item["name"], "_", ten_id, ".json"),
                                       "url": "{0}".format(ten_item["page_url"]), "orig_url": "{0}{1}".format(ten_item["page_url"], livediscover_url),
                                       "pageurl": "{0}{1}?pageSize={2}".format(ten_item['page_url'], livediscover_url,
                                                                                  page_size), 
                                       "name": "{0}".format(ten_item["name"]),
                                       "headers": ten_item["headers"]}
        return tenant_url_data
    else:
        # this is return if an invalid api variable is supplied
        logging.error("The {0} API does not appear to exist.".format(api))
        exit(1)


def build_json_data(ep_item_data, json_items, event_count):
    # from the item data retrieved build json with event count
    for item in ep_item_data:
        event_count += 1
        json_items[event_count] = item
    return json_items, event_count


def calculate_from_to(days, poll_date):
    # dependent on whether polling has been selected and the number of days passed to calculate the from/to dates
    def calc_from(polling_date, delta_calc, time_now):
        # This calculates the from date based on the information in the poll config date passed
        if polling_date:
            less_delta = (polling_date - delta_calc)
            start = '{0:%Y-%m-%dT%H:%M:%S.%fZ}'.format(less_delta)
            # Need to remove milliseconds from the calculated date so it is accepted by Sophos Central API
            start = start[:-4]
            start = "{0}Z".format(start)
            return start
        else:
            # calculate the from date by the value of delta
            less_delta = (time_now - delta_calc)
            pre_start = datetime.combine(less_delta, datetime.min.time())
            new_start = pre_start.strftime('%Y-%m-%dT%H:%M:%S.%f')
            # Need to remove milliseconds from the calculated date so it is accepted by Sophos Central API
            new_start = new_start[:-3]
            new_start = "{0}Z".format(new_start)
            return new_start

    # calculate the datetime now
    now = datetime.utcnow()
    time = "days"
    # delta is calculated by the number of days passed
    delta = timedelta(**{time: days})
    logging.debug("now={0}, time={1}, delta={2}".format(now, time, delta))
    # send to calc_from function to get formatted datetime
    from_str = calc_from(poll_date, delta, now)

    # calculate the to date from now.
    new_to = now.strftime('%Y-%m-%dT%H:%M:%S.%f')
    # Need to remove milliseconds from the calculated date so it is accepted by Sophos Central API
    new_to = new_to[:-3]
    to_str = "{0}Z".format(new_to)
    logging.debug("to_str={0}, from_str={1}".format(to_str, from_str))
    return to_str, from_str


def validate_page_size(page_size, api):
    # The sophos central api has a number page size which can be retrieved. The max is set in the
    # sophos_central_api_config.py. Below verifies that the sizes set in sophos_config.ini are within limits
    if api == "endpoint":
        # Verify the page sizes for the endpoint api
        logging.info("Verifying page size requested")
        if page_size == "":
            # if no value is set then a default value is passed for this variable
            logging.info("No value set in the config. Apply default.")
            page_size = 50
        elif int(page_size) > api_conf.max_inv_page:
            # Returns an error if the size of the page passed is greater than the max
            logging.error("Inventory page size has a max of: {0}".format(api_conf.max_inv_page))
            logging.error("Please ensure that the value in config.ini is less than or equal to this")
            exit(1)
        elif int(page_size) <= api_conf.max_inv_page:
            # Continues if the page size set is less than or equal to the max
            logging.info("Applying setting from config. Setting is less than or equal to the maximum allowed")
            pass
    elif api == "common":
        # Verify the page sizes for the common api
        if page_size == "":
            # set a limit if no page_size is passed
            logging.info("No value set in the config. Apply default.")
            page_size = 50
        elif int(page_size) <= api_conf.max_alerts_page:
            # Continues if the page size set is less than or equal to the max
            logging.info("Applying setting from config. Setting is less than or equal to the maximum allowed")
            pass
        elif int(page_size) > api_conf.max_alerts_page:
            # Returns an error if the size of the page passed is greater than the max
            logging.error("Alerts page size has a max of: {0}".format(api_conf.max_alerts_page))
            logging.error("Please ensure that the value in config.ini is less than or equal to this")
            exit(1)
    elif api == "local-sites":
        # Verify the page sizes for the local sites api
        if page_size == "":
            # set a limit if no page_size is passed
            logging.info("No value set in the config. Apply default.")
            page_size = 50
        elif int(page_size) <= api_conf.max_localsite_page:
            # Continues if the page size set is less than or equal to the max
            logging.info("Applying setting from config. Setting is less than or equal to the maximum allowed")
            pass
        elif int(page_size) > api_conf.max_localsite_page:
            # Returns an error if the size of the page passed is greater than the max
            logging.error("Local Sites page size has a max of: {0}".format(api_conf.max_localsite_page))
            logging.error("Please ensure that the value in config.ini is less than or equal to this")
            exit(1)

    return page_size


def get_file_location(process_path):
    dir_name = path.dirname(path.abspath(__file__))
    final_path = "{0}{1}".format(dir_name, process_path)
    return final_path


def open_tmp_json(file):
    # Open json file if available
    try:
        tmp_path = api_conf.temp_path
        final_tmp_path = get_file_location(tmp_path)
        with open(os.path.join(final_tmp_path, file), "r", encoding='utf-8') as tmp_file:
            file = tmp_file.read()

        file_data = json.loads(q_data)
    except IOError:
        logging.error("Unable to open file: {0}".format(final_tmp_path))

    return file_data


def clean_up():
    tmp_path = api_conf.temp_path
    final_tmp_path = api_utils.get_file_location(tmp_path)
    if not os.path.exists(final_tmp_path):
        logging.info("Nothing to clean up")
    else:
        logging.info("Deleting: {0}".format(final_tmp_path))
        try:
            shutil.rmtree(final_tmp_path)
        except OSError as err:
            logging.error("Error: {0} - {1}".format(err.filename, err.strerror))
        else:
            logging.info("{0} dir an contents successfully deleted".format(final_tmp_path))
